@@ -24,13 +24,13 @@ import (
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	"sigs.k8s.io/controller-runtime/pkg/client/config"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
-	"sigs.k8s.io/controller-runtime/pkg/runtime/signals"
+	"sigs.k8s.io/controller-runtime/pkg/manager/signals"
 
 	oappsv1 "github.com/openshift/api/apps/v1"
 	orbacv1 "github.com/openshift/api/authorization/v1"
 	_ "github.com/openshift/generic-admission-server/pkg/cmd"
 
-	"github.com/openshift/hive/pkg/apis"
+	"github.com/openshift/hive/apis"
 	"github.com/openshift/hive/pkg/operator"
 	"github.com/openshift/hive/pkg/operator/hive"
 	utillogrus "github.com/openshift/hive/pkg/util/logrus"
@@ -40,9 +40,9 @@ import (
 const (
 	defaultLogLevel             = "info"
 	leaderElectionConfigMap     = "hive-operator-leader"
-	leaderElectionLeaseDuration = "360s"
-	leaderElectionRenewDeadline = "270s"
-	leaderElectionRetryPeriod   = "90s"
+	leaderElectionLeaseDuration = "120s"
+	leaderElectionRenewDeadline = "90s"
+	leaderElectionRetryPeriod   = "30s"
 )
 
 type controllerManagerOptions struct {
@@ -108,7 +108,7 @@ func newRootCommand() *cobra.Command {
 			run := func(ctx context.Context) {
 				// Create a new Cmd to provide shared dependencies and start components
 				mgr, err := manager.New(cfg, manager.Options{
-					MetricsBindAddress: "0",
+					MetricsBindAddress: ":2112",
 					Logger:             utillogrus.NewLogr(log.StandardLogger()),
 				})
 				if err != nil {
