@@ -13,7 +13,7 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/utils/pointer"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	"github.com/openshift/hive/pkg/constants"
 )
 
@@ -78,6 +78,10 @@ type Builder struct {
 	// ServingCertKey is the contents of a key for the ServingCert.
 	ServingCertKey string
 
+	// CredentailsMode is the Cloud Credential Operator mode to force in the generated install-config.
+	// Typically left unset for the default ('Mint' mode), or set to 'Manual'.
+	CredentialsMode string
+
 	// Adopt is a flag indicating we're adopting a pre-existing cluster.
 	Adopt bool
 
@@ -124,6 +128,15 @@ type Builder struct {
 
 	// InstallConfig Secret to be used as template for deployment install-config
 	InstallConfigTemplate string
+
+	// CentralMachineManagement
+	CentralMachineManagement bool
+
+	// BoundServiceAccountSigningKey is the private key used to sign ServiceAccounts. Primarily used for provisioning clusters that use AWS Security Token Service.
+	BoundServiceAccountSigningKey string
+
+	// PublishStrategy defines the publishing strategy for the install-config.
+	PublishStrategy string
 }
 
 // Validate ensures that the builder's fields are logically configured and usable to generate the cluster resources.
@@ -214,11 +227,26 @@ func (o *Builder) Build() ([]runtime.Object, error) {
 		allObjects = append(allObjects, cloudCredsSecret)
 	}
 
-	cloudCertificatesSecret := o.CloudBuilder.generateCloudCertificatesSecret(o)
-	if cloudCertificatesSecret != nil {
-		allObjects = append(allObjects, cloudCertificatesSecret)
+	if len(o.BoundServiceAccountSigningKey) > 0 {
+		allObjects = append(allObjects, &corev1.Secret{
+			TypeMeta: metav1.TypeMeta{
+				Kind:       "Secret",
+				APIVersion: corev1.SchemeGroupVersion.String(),
+			},
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      o.getBoundServiceAccountSigningKeySecretName(),
+				Namespace: o.Namespace,
+			},
+			Type: corev1.SecretTypeOpaque,
+			StringData: map[string]string{
+				constants.BoundServiceAccountSigningKeyFile: o.BoundServiceAccountSigningKey,
+			},
+		})
 	}
 
+	additionalCloudObjects := o.CloudBuilder.GenerateCloudObjects(o)
+	allObjects = append(allObjects, additionalCloudObjects...)
+
 	if o.InstallerManifests != nil {
 		allObjects = append(allObjects, o.generateInstallerManifestsConfigMap())
 	}
@@ -318,7 +346,19 @@ func (o *Builder) generateClusterDeployment() *hivev1.ClusterDeployment {
 		cd.Spec.Provisioning.ImageSetRef = &hivev1.ClusterImageSetReference{Name: o.ImageSet}
 	}
 
-	cd.Spec.Provisioning.InstallConfigSecretRef = corev1.LocalObjectReference{Name: o.getInstallConfigSecretName()}
+	if o.CentralMachineManagement {
+		cd.Spec.MachineManagement = &hivev1.MachineManagement{
+			Central: &hivev1.CentralMachineManagement{},
+		}
+	}
+
+	if len(o.BoundServiceAccountSigningKey) > 0 {
+		cd.Spec.BoundServiceAccountSignkingKeySecretRef = &corev1.LocalObjectReference{
+			Name: o.getBoundServiceAccountSigningKeySecretName(),
+		}
+	}
+
+	cd.Spec.Provisioning.InstallConfigSecretRef = &corev1.LocalObjectReference{Name: o.getInstallConfigSecretName()}
 	cd.Spec.Platform = o.CloudBuilder.GetCloudPlatform(o)
 
 	return cd
@@ -360,6 +400,7 @@ func (o *Builder) generateInstallConfigSecret() (*corev1.Secret, error) {
 			},
 		},
 		AdditionalTrustBundle: o.AdditionalTrustBundle,
+		Publish:               installertypes.PublishingStrategy(o.PublishStrategy),
 	}
 
 	o.CloudBuilder.addInstallConfigPlatform(o, installConfig)
@@ -391,6 +432,9 @@ func (o *Builder) mergeInstallConfigTemplate() (*corev1.Secret, error) {
 		return nil, fmt.Errorf("Error parsing installconfigtemplate: %s", err.Error())
 	}
 	ic.BaseDomain = o.BaseDomain
+	if ic.MetaData == nil {
+		ic.MetaData = &metav1.ObjectMeta{}
+	}
 	ic.MetaData.Name = o.Name
 
 	d, err := yaml.Marshal(ic)
@@ -565,13 +609,18 @@ func (o *Builder) GetPullSecretSecretName() string {
 	return fmt.Sprintf("%s-pull-secret", o.Name)
 }
 
+func (o *Builder) getBoundServiceAccountSigningKeySecretName() string {
+	return o.Name + "-sa-signing-key"
+}
+
 // CloudBuilder interface exposes the functions we will use to set cloud specific portions of the cluster's resources.
 type CloudBuilder interface {
 	addMachinePoolPlatform(o *Builder, mp *hivev1.MachinePool)
 	addInstallConfigPlatform(o *Builder, ic *installertypes.InstallConfig)
-	generateCloudCertificatesSecret(o *Builder) *corev1.Secret
 
 	GetCloudPlatform(o *Builder) hivev1.Platform
 	CredsSecretName(o *Builder) string
 	GenerateCredentialsSecret(o *Builder) *corev1.Secret
+	// GenerateCloudObjects returns any additional resources needed for a particular cloud provider.
+	GenerateCloudObjects(o *Builder) []runtime.Object
 }
