@@ -26,8 +26,8 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	apihelpers "github.com/openshift/hive/pkg/apis/helpers"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	apihelpers "github.com/openshift/hive/apis/helpers"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	"github.com/openshift/hive/pkg/clusterresource"
 	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
@@ -48,6 +48,12 @@ const (
 var (
 	// controllerKind contains the schema.GroupVersionKind for this controller type.
 	controllerKind = hivev1.SchemeGroupVersion.WithKind("ClusterPool")
+
+	// clusterPoolConditions are the cluster pool conditions controlled or initialized by cluster pool controller
+	clusterPoolConditions = []hivev1.ClusterPoolConditionType{
+		hivev1.ClusterPoolMissingDependenciesCondition,
+		hivev1.ClusterPoolCapacityAvailableCondition,
+	}
 )
 
 // Add creates a new ClusterPool Controller and adds it to the Manager with default RBAC. The Manager will set fields on the Controller
@@ -96,22 +102,20 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterPool, concurrentReconc
 	}
 
 	// Watch for changes to ClusterClaims
-	enqueuePoolForClaim := &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(
-			func(o handler.MapObject) []reconcile.Request {
-				claim, ok := o.Object.(*hivev1.ClusterClaim)
-				if !ok {
-					return nil
-				}
-				return []reconcile.Request{{
-					NamespacedName: types.NamespacedName{
-						Namespace: claim.Namespace,
-						Name:      claim.Spec.ClusterPoolName,
-					},
-				}}
-			},
-		),
-	}
+	enqueuePoolForClaim := handler.EnqueueRequestsFromMapFunc(
+		func(o client.Object) []reconcile.Request {
+			claim, ok := o.(*hivev1.ClusterClaim)
+			if !ok {
+				return nil
+			}
+			return []reconcile.Request{{
+				NamespacedName: types.NamespacedName{
+					Namespace: claim.Namespace,
+					Name:      claim.Spec.ClusterPoolName,
+				},
+			}}
+		},
+	)
 	if err := c.Watch(&source.Kind{Type: &hivev1.ClusterClaim{}}, enqueuePoolForClaim); err != nil {
 		return err
 	}
@@ -119,9 +123,8 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterPool, concurrentReconc
 	// Watch for changes to the hive cluster pool admin RoleBindings
 	if err := c.Watch(
 		&source.Kind{Type: &rbacv1.RoleBinding{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: requestsForRBACResources(r.Client, r.logger),
-		},
+		handler.EnqueueRequestsFromMapFunc(
+			requestsForRBACResources(r.Client, r.logger)),
 	); err != nil {
 		return err
 	}
@@ -129,9 +132,8 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterPool, concurrentReconc
 	// Watch for changes to the hive-cluster-pool-admin-binding RoleBinding
 	if err := c.Watch(
 		&source.Kind{Type: &rbacv1.RoleBinding{}},
-		&handler.EnqueueRequestsFromMapFunc{
-			ToRequests: requestsForCDRBACResources(r.Client, clusterPoolAdminRoleBindingName, r.logger),
-		},
+		handler.EnqueueRequestsFromMapFunc(
+			requestsForCDRBACResources(r.Client, clusterPoolAdminRoleBindingName, r.logger)),
 	); err != nil {
 		return err
 	}
@@ -139,12 +141,12 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterPool, concurrentReconc
 	return nil
 }
 
-func requestsForCDRBACResources(c client.Client, resourceName string, logger log.FieldLogger) handler.ToRequestsFunc {
-	return func(o handler.MapObject) []reconcile.Request {
-		if o.Meta.GetName() != resourceName {
+func requestsForCDRBACResources(c client.Client, resourceName string, logger log.FieldLogger) handler.MapFunc {
+	return func(o client.Object) []reconcile.Request {
+		if o.GetName() != resourceName {
 			return nil
 		}
-		clusterName := o.Meta.GetNamespace()
+		clusterName := o.GetNamespace()
 		cd := &hivev1.ClusterDeployment{}
 		if err := c.Get(context.Background(), client.ObjectKey{Namespace: clusterName, Name: clusterName}, cd); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to get ClusterDeployment for RBAC resource")
@@ -158,9 +160,9 @@ func requestsForCDRBACResources(c client.Client, resourceName string, logger log
 	}
 }
 
-func requestsForRBACResources(c client.Client, logger log.FieldLogger) handler.ToRequestsFunc {
-	return func(o handler.MapObject) []reconcile.Request {
-		binding, ok := o.Object.(*rbacv1.RoleBinding)
+func requestsForRBACResources(c client.Client, logger log.FieldLogger) handler.MapFunc {
+	return func(o client.Object) []reconcile.Request {
+		binding, ok := o.(*rbacv1.RoleBinding)
 		if !ok {
 			return nil
 		}
@@ -169,7 +171,7 @@ func requestsForRBACResources(c client.Client, logger log.FieldLogger) handler.T
 		}
 
 		cpList := &hivev1.ClusterPoolList{}
-		if err := c.List(context.Background(), cpList, client.InNamespace(o.Meta.GetNamespace())); err != nil {
+		if err := c.List(context.Background(), cpList, client.InNamespace(o.GetNamespace())); err != nil {
 			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to list cluster pools for RBAC resource")
 			return nil
 		}
@@ -198,7 +200,7 @@ type ReconcileClusterPool struct {
 
 // Reconcile reads the state of the ClusterPool, checks if we currently have enough ClusterDeployments waiting, and
 // attempts to reach the desired state if not.
-func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.Result, error) {
+func (r *ReconcileClusterPool) Reconcile(ctx context.Context, request reconcile.Request) (reconcile.Result, error) {
 	logger := controllerutils.BuildControllerLogger(ControllerName, "clusterPool", request.NamespacedName)
 	logger.Infof("reconciling cluster pool")
 	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
@@ -218,6 +220,18 @@ func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.R
 		return reconcile.Result{}, err
 	}
 
+	// Initialize cluster pool conditions if not set
+	newConditions := controllerutils.InitializeClusterPoolConditions(clp.Status.Conditions, clusterPoolConditions)
+	if len(newConditions) > len(clp.Status.Conditions) {
+		clp.Status.Conditions = newConditions
+		logger.Infof("initializing cluster pool conditions")
+		if err := r.Status().Update(context.TODO(), clp); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to update cluster pool status")
+			return reconcile.Result{}, err
+		}
+		return reconcile.Result{}, nil
+	}
+
 	// If the pool is deleted, clear finalizer once all ClusterDeployments have been deleted.
 	if clp.DeletionTimestamp != nil {
 		return reconcile.Result{}, r.reconcileDeletedPool(clp, logger)
@@ -552,6 +566,7 @@ func (r *ReconcileClusterPool) createCluster(
 		PullSecret:            pullSecret,
 		CloudBuilder:          cloudBuilder,
 		Labels:                clp.Spec.Labels,
+		Annotations:           clp.Spec.Annotations,
 		InstallConfigTemplate: installConfigTemplate,
 		SkipMachinePools:      clp.Spec.SkipMachinePools,
 	}
@@ -581,7 +596,7 @@ func (r *ReconcileClusterPool) createCluster(
 	}
 	// Create the resources.
 	for _, obj := range objs {
-		if err := r.Client.Create(context.Background(), obj); err != nil {
+		if err := r.Client.Create(context.Background(), obj.(client.Object)); err != nil {
 			r.expectations.CreationObserved(poolKey)
 			return err
 		}
@@ -825,11 +840,18 @@ func (r *ReconcileClusterPool) getPullSecret(pool *hivev1.ClusterPool, logger lo
 func (r *ReconcileClusterPool) createCloudBuilder(pool *hivev1.ClusterPool, logger log.FieldLogger) (clusterresource.CloudBuilder, error) {
 	switch platform := pool.Spec.Platform; {
 	case platform.AWS != nil:
-		credsSecret, err := r.getCredentialsSecret(pool, platform.AWS.CredentialsSecretRef.Name, logger)
-		if err != nil {
-			return nil, err
+		var cloudBuilder *clusterresource.AWSCloudBuilder
+		switch {
+		case platform.AWS.CredentialsSecretRef.Name != "":
+			credsSecret, err := r.getCredentialsSecret(pool, platform.AWS.CredentialsSecretRef.Name, logger)
+			if err != nil {
+				return nil, err
+			}
+			cloudBuilder = clusterresource.NewAWSCloudBuilderFromSecret(credsSecret)
+		case platform.AWS.CredentialsAssumeRole != nil:
+			cloudBuilder = clusterresource.NewAWSCloudBuilderFromAssumeRole(platform.AWS.CredentialsAssumeRole)
 		}
-		cloudBuilder := clusterresource.NewAWSCloudBuilderFromSecret(credsSecret)
+
 		cloudBuilder.Region = platform.AWS.Region
 		return cloudBuilder, nil
 	case platform.GCP != nil:
