@@ -20,7 +20,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	"github.com/openshift/hive/pkg/constants"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/remoteclient"
@@ -91,7 +91,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		cd                *hivev1.ClusterDeployment
 		srcResources      []runtime.Object
 		destResources     []runtime.Object
-		expectedResources []runtime.Object
+		expectedResources []client.Object
 	}{
 		{
 			name: "no relocation",
@@ -99,11 +99,14 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "only clusterdeployment",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -119,7 +122,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			destResources: []runtime.Object{
 				cdBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				cdBuilder.Build(),
 			},
 		},
@@ -136,7 +139,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					},
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				cdBuilder.Build(
 					func(cd *hivev1.ClusterDeployment) {
 						cd.Spec.ClusterMetadata = &hivev1.ClusterMetadata{}
@@ -157,7 +160,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			destResources: []runtime.Object{
 				cdBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				cdBuilder.Build(),
 			},
 		},
@@ -172,7 +175,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testcd.Generic(testgeneric.WithResourceVersion("some-rv")),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				cdBuilder.Build(
 					testcd.Generic(testgeneric.WithResourceVersion("some-rv")),
 				),
@@ -180,14 +183,17 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "existing namespace",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 			},
 			destResources: []runtime.Object{
 				namespaceBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -196,12 +202,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "single dependent",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(testsecret.WithDataKeyValue("test-key", []byte("test-data"))),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -211,7 +220,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "multiple dependents",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(
@@ -223,7 +235,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testsecret.WithDataKeyValue("test-key-2", []byte("test-data-2")),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -240,7 +252,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "dependent in other namespace",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(
@@ -253,7 +268,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testsecret.WithDataKeyValue("test-key-2", []byte("test-data-2")),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -266,7 +281,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "existing dependent",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(testsecret.WithDataKeyValue("test-key", []byte("test-data"))),
@@ -274,7 +292,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			destResources: []runtime.Object{
 				secretBuilder.Build(testsecret.WithDataKeyValue("test-key", []byte("test-data"))),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -284,7 +302,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "out-of-date dependent",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(testsecret.WithDataKeyValue("test-key", []byte("test-data"))),
@@ -292,7 +313,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			destResources: []runtime.Object{
 				secretBuilder.Build(testsecret.WithDataKeyValue("test-key", []byte("other-data"))),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -302,7 +323,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "existing dependent with different status",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				mpBuilder.Build(
@@ -314,7 +338,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			destResources: []runtime.Object{
 				mpBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -324,7 +348,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "existing dependent with different instance-specific metadata",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(
@@ -337,7 +364,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testsecret.WithDataKeyValue("test-key", []byte("test-data")),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -350,7 +377,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "ignore service account token",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(
@@ -358,7 +388,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testsecret.WithDataKeyValue("test-key", []byte("test-data")),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -367,7 +397,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "ignore secret owned by service account token",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				secretBuilder.Build(
@@ -386,7 +419,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					},
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -395,12 +428,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "configmap",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				cmBuilder.Build(testcm.WithDataKeyValue("test-key", "test-data")),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -410,12 +446,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "machinepool",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				mpBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -425,12 +464,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "syncset",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				ssBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -440,12 +482,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "syncidentityprovider",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				sipBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -455,12 +500,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "dnszone",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				dnsZoneBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -472,12 +520,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "non-child dnszone",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				dnsZoneBuilder.Build(testdnszone.Generic(testgeneric.WithName("other-dnszone"))),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -486,12 +537,15 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "non-dependent",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				jobBuilder.Build(),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -500,7 +554,10 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 		},
 		{
 			name: "full",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
 				cdBuilder.Build(testcd.WithName("other-cluster-deployment")),
@@ -633,7 +690,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					testsip.ForIdentities("other-user"),
 				),
 			},
-			expectedResources: []runtime.Object{
+			expectedResources: []client.Object{
 				namespaceBuilder.Build(),
 				cdBuilder.Build(
 					testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -725,7 +782,7 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 					return mockRemoteClientBuilder
 				},
 			}
-			_, err := reconciler.Reconcile(reconcile.Request{
+			_, err := reconciler.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      cdName,
 					Namespace: namespace,
@@ -734,11 +791,8 @@ func TestReconcileClusterRelocate_Reconcile_Movement(t *testing.T) {
 			require.NoError(t, err, "unexpected error during reconcile")
 
 			for _, obj := range tc.expectedResources {
-				objKey, err := client.ObjectKeyFromObject(obj)
-				if !assert.NoError(t, err, "unexpected error getting object key") {
-					continue
-				}
-				destObj := reflect.New(reflect.TypeOf(obj).Elem()).Interface().(runtime.Object)
+				objKey := client.ObjectKeyFromObject(obj)
+				destObj := reflect.New(reflect.TypeOf(obj).Elem()).Interface().(client.Object)
 				err = destClient.Get(context.Background(), objKey, destObj)
 				if !assert.NoError(t, err, "unexpected error getting destination object") {
 					continue
@@ -782,8 +836,11 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 		validate                          func(t *testing.T, cd *hivev1.ClusterDeployment)
 	}{
 		{
-			name:    "fresh clusterdeployment",
-			cd:      cdBuilder.Build(),
+			name: "fresh clusterdeployment",
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			dnsZone: dnsZoneBuilder.Build(),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
@@ -793,7 +850,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 		},
 		{
 			name: "clusterdeployment with dnszone already relocating",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateOutgoing)),
 			),
@@ -807,6 +867,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			name: "switch relocates",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateOutgoing)),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.RelocationFailedCondition,
+					Status: corev1.ConditionUnknown,
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateOutgoing)),
@@ -818,8 +882,11 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			expectedDeletionTimestamp: true,
 		},
 		{
-			name:    "multiple relocates",
-			cd:      cdBuilder.Build(),
+			name: "multiple relocates",
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			dnsZone: dnsZoneBuilder.Build(),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
@@ -836,6 +903,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			name: "multiple relocates when already relocating",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateOutgoing)),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.RelocationFailedCondition,
+					Status: corev1.ConditionUnknown,
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateOutgoing)),
@@ -855,6 +926,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			name: "already relocated clusterdeployment",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateComplete)),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.RelocationFailedCondition,
+					Status: corev1.ConditionUnknown,
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation(crName, hivev1.RelocateComplete)),
@@ -866,8 +941,11 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			expectedDeletionTimestamp: true,
 		},
 		{
-			name:    "already moved clusterdeployment",
-			cd:      cdBuilder.Build(),
+			name: "already moved clusterdeployment",
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			dnsZone: dnsZoneBuilder.Build(),
 			srcResources: []runtime.Object{
 				crBuilder.Build(),
@@ -880,7 +958,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 		},
 		{
 			name: "already moved clusterdeployment with dnszone already completed",
-			cd:   cdBuilder.Build(),
+			cd: cdBuilder.Build(testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+				Type:   hivev1.RelocationFailedCondition,
+				Status: corev1.ConditionUnknown,
+			})),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation(crName, hivev1.RelocateComplete)),
 			),
@@ -898,6 +979,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			cd: cdBuilder.Build(
 				func(cd *hivev1.ClusterDeployment) {
 					cd.Spec.BaseDomain = "test-domain"
+					cd.Status.Conditions = append(cd.Status.Conditions, hivev1.ClusterDeploymentCondition{
+						Status: corev1.ConditionUnknown,
+						Type:   hivev1.RelocationFailedCondition,
+					})
 				},
 			),
 			dnsZone: dnsZoneBuilder.Build(),
@@ -920,23 +1005,44 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 			name: "incoming",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateIncoming)),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.RelocationFailedCondition,
+					Status: corev1.ConditionUnknown,
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateIncoming)),
 			),
+			expectedRelocationFailedCondition: &hivev1.ClusterDeploymentCondition{
+				Status: corev1.ConditionFalse,
+				Reason: "NoMatchingRelocates",
+			},
 		},
 		{
 			name: "incoming with dnszone already released",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation("other-relocate", hivev1.RelocateIncoming)),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.RelocationFailedCondition,
+					Status: corev1.ConditionUnknown,
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(),
+			expectedRelocationFailedCondition: &hivev1.ClusterDeploymentCondition{
+				Status: corev1.ConditionFalse,
+				Reason: "NoMatchingRelocates",
+			},
 		},
 		{
 			name: "clusterdeployment deleted while outgoing",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateOutgoing)),
 				testcd.Generic(testgeneric.Deleted()),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Status: corev1.ConditionFalse,
+					Type:   hivev1.RelocationFailedCondition,
+					Reason: "DontUpdateCondition",
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation(crName, hivev1.RelocateOutgoing)),
@@ -945,12 +1051,21 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 				crBuilder.Build(),
 			},
 			expectedDeletionTimestamp: true,
+			expectedRelocationFailedCondition: &hivev1.ClusterDeploymentCondition{
+				Status: corev1.ConditionFalse,
+				Reason: "DontUpdateCondition",
+			},
 		},
 		{
 			name: "clusterdeployment deleted while incoming",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
 				testcd.Generic(testgeneric.Deleted()),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Status: corev1.ConditionFalse,
+					Type:   hivev1.RelocationFailedCondition,
+					Reason: "DontUpdateCondition",
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation(crName, hivev1.RelocateIncoming)),
@@ -959,12 +1074,21 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 				crBuilder.Build(),
 			},
 			expectedDeletionTimestamp: true,
+			expectedRelocationFailedCondition: &hivev1.ClusterDeploymentCondition{
+				Status: corev1.ConditionFalse,
+				Reason: "DontUpdateCondition",
+			},
 		},
 		{
 			name: "clusterdeployment deleted while outgoing",
 			cd: cdBuilder.Build(
 				testcd.Generic(withRelocateAnnotation(crName, hivev1.RelocateOutgoing)),
 				testcd.Generic(testgeneric.Deleted()),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Status: corev1.ConditionFalse,
+					Type:   hivev1.RelocationFailedCondition,
+					Reason: "DontUpdateCondition",
+				}),
 			),
 			dnsZone: dnsZoneBuilder.Build(
 				testdnszone.Generic(withRelocateAnnotation(crName, hivev1.RelocateOutgoing)),
@@ -973,6 +1097,10 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 				crBuilder.Build(),
 			},
 			expectedDeletionTimestamp: true,
+			expectedRelocationFailedCondition: &hivev1.ClusterDeploymentCondition{
+				Status: corev1.ConditionFalse,
+				Reason: "DontUpdateCondition",
+			},
 		},
 	}
 	for _, tc := range cases {
@@ -1004,7 +1132,7 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 					return mockRemoteClientBuilder
 				},
 			}
-			_, err := reconciler.Reconcile(reconcile.Request{
+			_, err := reconciler.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      cdName,
 					Namespace: namespace,
@@ -1053,7 +1181,8 @@ func TestReconcileClusterRelocate_Reconcile_RelocateStatus(t *testing.T) {
 					assert.Equal(t, tc.expectedRelocationFailedCondition.Reason, cond.Reason, "unexpected condition reason")
 				}
 			} else {
-				assert.Nil(t, cond, "unexpected relocation failed condition")
+				assert.Equal(t, corev1.ConditionFalse, cond.Status, "unexpected condition status")
+				assert.Equal(t, "MoveSuccessful", cond.Reason, "unexpected condition reason")
 			}
 
 			if tc.validate != nil {
