@@ -27,8 +27,8 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/remoteclient"
 	remoteclientmock "github.com/openshift/hive/pkg/remoteclient/mock"
@@ -130,7 +130,7 @@ func (rt *reconcileTest) run(t *testing.T) {
 
 	startTime := time.Now()
 	timeSinceOrigLeaseRenewTime := time.Since(origLeaseRenewTime.Time)
-	result, err := rt.r.Reconcile(reconcileRequest)
+	result, err := rt.r.Reconcile(context.TODO(), reconcileRequest)
 	require.NoError(t, err, "unexpected error from Reconcile")
 	endTime := time.Now()
 	startTime = startTime.Truncate(time.Second)
@@ -256,7 +256,7 @@ func TestReconcileClusterSync_NewClusterDeployment(t *testing.T) {
 			Name:      testCDName,
 		},
 	}
-	result, err := rt.r.Reconcile(reconcileRequest)
+	result, err := rt.r.Reconcile(context.TODO(), reconcileRequest)
 	require.NoError(t, err, "unexpected error from Reconcile")
 	assert.Equal(t, result, reconcile.Result{Requeue: true}, "unexpected result from reconcile")
 	err = rt.c.Get(context.Background(), client.ObjectKey{Namespace: testNamespace, Name: testLeaseName}, &hiveintv1alpha1.ClusterSyncLease{})
@@ -288,6 +288,18 @@ func TestReconcileClusterSync_NoWorkToDo(t *testing.T) {
 				}),
 			),
 		},
+		{
+			name: "uninstalled and unreachable unknown",
+			cd: testcd.FullBuilder(testNamespace, testCDName, scheme).
+				GenericOptions(
+					testgeneric.WithUID(testCDUID),
+				).
+				Options(
+					testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+						Type:   hivev1.UnreachableCondition,
+						Status: corev1.ConditionUnknown,
+					})).Build(),
+		},
 		{
 			name: "syncset pause",
 			cd:   cdBuilder(scheme).GenericOptions(testgeneric.WithAnnotation(constants.SyncsetPauseAnnotation, "true")).Build(),
@@ -885,16 +897,19 @@ func TestReconcileClusterSync_ResourceRemovedFromSyncSet(t *testing.T) {
 		name                     string
 		includeResourcesToDelete bool
 		expectDelete             bool
+		resourceApplyMode        hivev1.SyncSetResourceApplyMode
 	}{
 		{
 			name:                     "upsert",
 			includeResourcesToDelete: false,
 			expectDelete:             false,
+			resourceApplyMode:        hivev1.UpsertResourceApplyMode,
 		},
 		{
 			name:                     "sync",
 			includeResourcesToDelete: true,
 			expectDelete:             true,
+			resourceApplyMode:        hivev1.SyncResourceApplyMode,
 		},
 	}
 	for _, tc := range cases {
@@ -907,6 +922,7 @@ func TestReconcileClusterSync_ResourceRemovedFromSyncSet(t *testing.T) {
 				testsyncset.ForClusterDeployments(testCDName),
 				testsyncset.WithGeneration(2),
 				testsyncset.WithResources(resourceToApply),
+				testsyncset.WithApplyMode(tc.resourceApplyMode),
 			)
 			existingSyncStatusBuilder := newSyncStatusBuilder("test-syncset").Options(
 				withTransitionInThePast(),
@@ -1728,6 +1744,7 @@ func TestReconcileClusterSync_DeleteErrorDoesNotBlockOtherDeletes(t *testing.T)
 					testsyncset.FullBuilder(testNamespace, "test-syncset", scheme).Build(
 						testsyncset.ForClusterDeployments(testCDName),
 						testsyncset.WithGeneration(2),
+						testsyncset.WithApplyMode(hivev1.SyncResourceApplyMode),
 					),
 				)
 			}
