@@ -3,6 +3,8 @@ package hibernation
 import (
 	"context"
 	"fmt"
+	"sort"
+	"strings"
 	"time"
 
 	"github.com/blang/semver/v4"
@@ -23,10 +25,10 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
@@ -39,10 +41,10 @@ const (
 
 	// stateCheckInterval is the time interval for polling
 	// whether a cluster's machines are stopped or are running
-	stateCheckInterval = 60 * time.Second
+	stateCheckInterval = 1 * time.Minute
 
 	// csrCheckInterval is the time interval for polling
-	// pending CSRs
+	// pending CertificateSigningRequests
 	csrCheckInterval = 30 * time.Second
 
 	// nodeCheckWaitTime is the minimum time to wait for a node
@@ -64,6 +66,12 @@ var (
 	// actuators is a list of available actuators for this controller
 	// It is populated via the RegisterActuator function
 	actuators []HibernationActuator
+
+	// clusterDeploymentHibernationConditions are the cluster deployment conditions controlled by
+	// hibernation controller
+	clusterDeploymentHibernationConditions = []hivev1.ClusterDeploymentConditionType{
+		hivev1.ClusterHibernatingCondition,
+	}
 )
 
 // Add creates a new Hibernation controller and adds it to the manager with default RBAC.
@@ -129,7 +137,7 @@ func AddToManager(mgr manager.Manager, r *hibernationReconciler, concurrentRecon
 }
 
 // Reconcile syncs a single ClusterDeployment
-func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result reconcile.Result, returnErr error) {
+func (r *hibernationReconciler) Reconcile(ctx context.Context, request reconcile.Request) (result reconcile.Result, returnErr error) {
 	cdLog := controllerutils.BuildControllerLogger(ControllerName, "clusterDeployment", request.NamespacedName)
 	cdLog.Info("reconciling cluster deployment")
 	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, cdLog)
@@ -155,21 +163,33 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 		return reconcile.Result{}, nil
 	}
 
+	// Initialize cluster deployment conditions if not present
+	newConditions := controllerutils.InitializeClusterDeploymentConditions(cd.Status.Conditions, clusterDeploymentHibernationConditions)
+	if len(newConditions) > len(cd.Status.Conditions) {
+		cd.Status.Conditions = newConditions
+		cdLog.Info("initializing hibernating controller conditions")
+		if err := r.Status().Update(context.TODO(), cd); err != nil {
+			cdLog.WithError(err).Log(controllerutils.LogLevel(err), "failed to update cluster deployment status")
+			return reconcile.Result{}, err
+		}
+	}
+
 	// If cluster is not installed, skip any processing
 	if !cd.Spec.Installed {
 		return reconcile.Result{}, nil
 	}
 
+	// set hibernating condition to false for unsupported clouds
+	if supported, msg := r.hibernationSupported(cd); !supported {
+		return r.setHibernatingCondition(cd, hivev1.UnsupportedHibernationReason, msg, corev1.ConditionFalse, cdLog)
+	}
+
 	shouldHibernate := cd.Spec.PowerState == hivev1.HibernatingClusterPowerState
 	hibernatingCondition := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition)
 
-	// Signal a problem if we should be hibernating or have requested hibernate after and the cluster does not support it or
+	// Signal a problem if we should be hibernating or have requested hibernate after and the
 	// SyncSets have not yet been applied.
 	if shouldHibernate || cd.Spec.HibernateAfter != nil {
-		if supported, msg := r.hibernationSupported(cd); !supported {
-			return r.setHibernatingCondition(cd, hivev1.UnsupportedHibernationReason, msg, corev1.ConditionFalse, cdLog)
-		}
-
 		// Determine if SyncSets have been applied
 		clusterSync := &hiveintv1alpha1.ClusterSync{}
 		err := r.Get(context.Background(), types.NamespacedName{Namespace: cd.Namespace, Name: cd.Name}, clusterSync)
@@ -187,7 +207,7 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 	}
 
 	// Clear any lingering unsupported hibernation condition
-	if hibernatingCondition != nil && hibernatingCondition.Reason == hivev1.UnsupportedHibernationReason {
+	if hibernatingCondition.Reason == hivev1.UnsupportedHibernationReason {
 		if supported, msg := r.hibernationSupported(cd); supported {
 			return r.setHibernatingCondition(cd, hivev1.RunningHibernationReason, msg, corev1.ConditionFalse, cdLog)
 		}
@@ -204,8 +224,8 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 		var isRunning bool
 
 		cond := controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition)
-		if cond == nil {
-			hibLog.Debug("cluster has no hibernating condition (never hibernated), using installed time")
+		if cond.Status == corev1.ConditionUnknown {
+			hibLog.Debug("cluster has never been hibernated, using installed time")
 			isRunning = true
 		} else if cond.Status == corev1.ConditionFalse {
 			runningSince = cond.LastTransitionTime.Time
@@ -220,6 +240,14 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 			if time.Now().After(expiry) {
 				hibLog.WithField("expiry", expiry).Debug("cluster has been running longer than hibernate-after duration, moving to hibernating powerState")
 				cd.Spec.PowerState = hivev1.HibernatingClusterPowerState
+				if controllerutils.IsFakeCluster(cd) {
+					controllerutils.SetClusterDeploymentCondition(cd.Status.Conditions,
+						hivev1.ClusterHibernatingCondition,
+						corev1.ConditionTrue,
+						hivev1.HibernatingHibernationReason,
+						"Fake cluster is stopped",
+						controllerutils.UpdateConditionNever)
+				}
 				err := r.Update(context.TODO(), cd)
 				if err != nil {
 					hibLog.WithError(err).Log(controllerutils.LogLevel(err), "error hibernating cluster")
@@ -245,11 +273,27 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 	}
 
 	if !shouldHibernate {
-		if hibernatingCondition == nil || hibernatingCondition.Status == corev1.ConditionFalse {
+		if hibernatingCondition.Status == corev1.ConditionUnknown || hibernatingCondition.Status == corev1.ConditionFalse {
 			return reconcile.Result{}, nil
 		}
 		switch hibernatingCondition.Reason {
 		case hivev1.StoppingHibernationReason, hivev1.HibernatingHibernationReason, hivev1.FailedToStartHibernationReason:
+			if controllerutils.IsFakeCluster(cd) {
+				cd.Spec.PowerState = hivev1.RunningClusterPowerState
+				if controllerutils.IsFakeCluster(cd) {
+					controllerutils.SetClusterDeploymentCondition(cd.Status.Conditions,
+						hivev1.ClusterHibernatingCondition,
+						corev1.ConditionFalse,
+						hivev1.RunningHibernationReason,
+						"Fake cluster is running",
+						controllerutils.UpdateConditionNever)
+				}
+				err := r.Update(context.TODO(), cd)
+				if err != nil {
+					cdLog.WithError(err).Log(controllerutils.LogLevel(err), "error resuming cluster")
+				}
+				return reconcile.Result{}, err
+			}
 			return r.startMachines(cd, cdLog)
 		case hivev1.ResumingHibernationReason:
 			return r.checkClusterResumed(cd, cdLog)
@@ -257,7 +301,8 @@ func (r *hibernationReconciler) Reconcile(request reconcile.Request) (result rec
 		return reconcile.Result{}, nil
 	}
 
-	if hibernatingCondition == nil || hibernatingCondition.Status == corev1.ConditionFalse ||
+	if (hibernatingCondition.Status == corev1.ConditionUnknown || hibernatingCondition.Status == corev1.ConditionFalse &&
+		hibernatingCondition.Reason != hivev1.UnsupportedHibernationReason) ||
 		hibernatingCondition.Reason == hivev1.ResumingHibernationReason ||
 		(cd.Spec.PowerState == hivev1.HibernatingClusterPowerState && hibernatingCondition.Reason == hivev1.FailedToStartHibernationReason) {
 		return r.stopMachines(cd, cdLog)
@@ -307,14 +352,29 @@ func (r *hibernationReconciler) checkClusterStopped(cd *hivev1.ClusterDeployment
 		logger.Warning("No compatible actuator found to check machine status")
 		return reconcile.Result{}, nil
 	}
-	stopped, err := actuator.MachinesStopped(cd, r.Client, logger)
+
+	stopped, remaining, err := actuator.MachinesStopped(cd, r.Client, logger)
 	if err != nil {
 		logger.WithError(err).Log(controllerutils.LogLevel(err), "Failed to check whether machines are stopped.")
 		return reconcile.Result{}, err
 	}
 	if !stopped {
+		// Ensure all machines have been stopped. Should have been handled already but we've seen VMs left in running state.
+		if err := actuator.StopMachines(cd, r.Client, logger); err != nil {
+			logger.WithError(err).Error("error stopping machines")
+			return reconcile.Result{}, err
+		}
+
+		sort.Strings(remaining) // we want to make sure the message is stable.
+		msg := fmt.Sprintf("Stopping cluster machines. Some machines have not yet stopped: %s", strings.Join(remaining, ","))
+		_, condErr := r.setHibernatingCondition(cd, hivev1.StoppingHibernationReason, msg, corev1.ConditionTrue, logger)
+		if condErr != nil {
+			return reconcile.Result{}, condErr
+		}
+
 		return reconcile.Result{RequeueAfter: stateCheckInterval}, nil
 	}
+
 	logger.Info("Cluster has stopped and is in hibernating state")
 	return r.setHibernatingCondition(cd, hivev1.HibernatingHibernationReason, "Cluster is stopped", corev1.ConditionTrue, logger)
 }
@@ -325,14 +385,29 @@ func (r *hibernationReconciler) checkClusterResumed(cd *hivev1.ClusterDeployment
 		logger.Warning("No compatible actuator found to check machine status")
 		return reconcile.Result{}, nil
 	}
-	running, err := actuator.MachinesRunning(cd, r.Client, logger)
+
+	running, remaining, err := actuator.MachinesRunning(cd, r.Client, logger)
 	if err != nil {
 		logger.WithError(err).Log(controllerutils.LogLevel(err), "Failed to check whether machines are running.")
 		return reconcile.Result{}, err
 	}
 	if !running {
+		// Ensure all machines have been started. Should have been handled already but we've seen VMs left in stopped state.
+		if err := actuator.StartMachines(cd, r.Client, logger); err != nil {
+			logger.WithError(err).Error("error starting machines")
+			return reconcile.Result{}, err
+		}
+
+		sort.Strings(remaining) // we want to make sure the message is stable.
+		msg := fmt.Sprintf("Starting cluster machines. Some machines are not yet running: %s", strings.Join(remaining, ","))
+		_, condErr := r.setHibernatingCondition(cd, hivev1.ResumingHibernationReason, msg, corev1.ConditionTrue, logger)
+		if condErr != nil {
+			return reconcile.Result{}, condErr
+		}
+
 		return reconcile.Result{RequeueAfter: stateCheckInterval}, nil
 	}
+
 	remoteClient, err := r.remoteClientBuilder(cd).Build()
 	if err != nil {
 		logger.WithError(err).Log(controllerutils.LogLevel(err), "Failed to connect to target cluster")
@@ -353,27 +428,14 @@ func (r *hibernationReconciler) checkClusterResumed(cd *hivev1.ClusterDeployment
 
 func (r *hibernationReconciler) setHibernatingCondition(cd *hivev1.ClusterDeployment, reason, message string, status corev1.ConditionStatus, logger log.FieldLogger) (result reconcile.Result, returnErr error) {
 	changed := false
-	if status == corev1.ConditionFalse && controllerutils.FindClusterDeploymentCondition(cd.Status.Conditions, hivev1.ClusterHibernatingCondition) == nil {
-		now := metav1.Now()
-		cd.Status.Conditions = append(cd.Status.Conditions, hivev1.ClusterDeploymentCondition{
-			Type:               hivev1.ClusterHibernatingCondition,
-			Status:             corev1.ConditionFalse,
-			Reason:             reason,
-			Message:            message,
-			LastProbeTime:      now,
-			LastTransitionTime: now,
-		})
-		changed = true
-	} else {
-		cd.Status.Conditions, changed = controllerutils.SetClusterDeploymentConditionWithChangeCheck(
-			cd.Status.Conditions,
-			hivev1.ClusterHibernatingCondition,
-			status,
-			reason,
-			message,
-			controllerutils.UpdateConditionIfReasonOrMessageChange,
-		)
-	}
+	cd.Status.Conditions, changed = controllerutils.SetClusterDeploymentConditionWithChangeCheck(
+		cd.Status.Conditions,
+		hivev1.ClusterHibernatingCondition,
+		status,
+		reason,
+		message,
+		controllerutils.UpdateConditionIfReasonOrMessageChange,
+	)
 
 	if reason == hivev1.SyncSetsNotAppliedReason {
 		defer func() {
