@@ -22,10 +22,11 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hiveintv1alpha1 "github.com/openshift/hive/pkg/apis/hiveinternal/v1alpha1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hiveintv1alpha1 "github.com/openshift/hive/apis/hiveinternal/v1alpha1"
+	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/controller/hibernation/mock"
 	"github.com/openshift/hive/pkg/remoteclient"
 	remoteclientmock "github.com/openshift/hive/pkg/remoteclient/mock"
@@ -80,11 +81,25 @@ func TestReconcile(t *testing.T) {
 			},
 		},
 		{
-			name: "cluster not installed",
+			name: "hibernation condition initialized",
 			cd:   cdBuilder.Options(o.notInstalled, o.shouldHibernate).Build(),
 			cs:   csBuilder.Build(),
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
-				require.Nil(t, getHibernatingCondition(cd))
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionUnknown, cond.Status)
+				assert.Equal(t, hivev1.InitializedConditionReason, cond.Reason)
+			},
+		},
+		{
+			name: "do not hibernate unsupported versions",
+			cd:   cdBuilder.Options(testcd.WithClusterVersion("4.3.11")).Build(),
+			cs:   csBuilder.Build(),
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, corev1.ConditionFalse, cond.Status)
+				assert.Equal(t, hivev1.UnsupportedHibernationReason, cond.Reason)
 			},
 		},
 		{
@@ -157,7 +172,7 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.shouldHibernate, o.stopping).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
+				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil, nil)
 			},
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				cond := getHibernatingCondition(cd)
@@ -171,13 +186,16 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.shouldHibernate, o.stopping).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(false, nil)
+				actuator.EXPECT().StopMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+				actuator.EXPECT().MachinesStopped(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).
+					Return(false, []string{"running-1", "pending-1", "stopping-1"}, nil)
 			},
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				cond := getHibernatingCondition(cd)
 				require.NotNil(t, cond)
 				assert.Equal(t, corev1.ConditionTrue, cond.Status)
 				assert.Equal(t, hivev1.StoppingHibernationReason, cond.Reason)
+				assert.Equal(t, "Stopping cluster machines. Some machines have not yet stopped: pending-1,running-1,stopping-1", cond.Message)
 			},
 		},
 		{
@@ -256,13 +274,16 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.resuming).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(false, nil)
+				actuator.EXPECT().StartMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).
+					Return(false, []string{"stopped-1", "pending-1"}, nil)
 			},
 			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
 				cond := getHibernatingCondition(cd)
 				require.NotNil(t, cond)
 				assert.Equal(t, corev1.ConditionTrue, cond.Status)
 				assert.Equal(t, hivev1.ResumingHibernationReason, cond.Reason)
+				assert.Equal(t, "Starting cluster machines. Some machines are not yet running: pending-1,stopped-1", cond.Message)
 			},
 		},
 		{
@@ -270,7 +291,7 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.resuming).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
+				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil, nil)
 			},
 			setupRemote: func(builder *remoteclientmock.MockBuilder) {
 				c := fake.NewFakeClientWithScheme(scheme, readyNodes()...)
@@ -288,7 +309,7 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.resuming).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
+				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil, nil)
 			},
 			setupRemote: func(builder *remoteclientmock.MockBuilder) {
 				fakeClient := fake.NewFakeClientWithScheme(scheme, unreadyNode()...)
@@ -308,7 +329,7 @@ func TestReconcile(t *testing.T) {
 			cd:   cdBuilder.Options(o.resuming).Build(),
 			cs:   csBuilder.Build(),
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil)
+				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(true, nil, nil)
 			},
 			setupRemote: func(builder *remoteclientmock.MockBuilder) {
 				fakeClient := fake.NewFakeClientWithScheme(scheme, unreadyNode()...)
@@ -341,6 +362,34 @@ func TestReconcile(t *testing.T) {
 				assert.Equal(t, "Hibernation capable", cond.Message)
 			},
 		},
+		{
+			name: "hibernate fake cluster",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(1*time.Hour),
+				testcd.InstalledTimestamp(time.Now().Add(-1*time.Hour)),
+				testcd.WithAnnotation(constants.HiveFakeClusterAnnotation, "true")),
+			cs: csBuilder.Build(),
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, hivev1.HibernatingHibernationReason, cond.Reason)
+				assert.Equal(t, corev1.ConditionTrue, cond.Status)
+				assert.Equal(t, "Fake cluster is stopped", cond.Message)
+			},
+		},
+		{
+			name: "start hibernated fake cluster",
+			cd:   cdBuilder.Options(o.hibernating, testcd.WithAnnotation(constants.HiveFakeClusterAnnotation, "true")).Build(),
+			cs:   csBuilder.Build(),
+			validate: func(t *testing.T, cd *hivev1.ClusterDeployment) {
+				assert.Equal(t, hivev1.RunningClusterPowerState, cd.Spec.PowerState)
+				cond := getHibernatingCondition(cd)
+				require.NotNil(t, cond)
+				assert.Equal(t, hivev1.RunningHibernationReason, cond.Reason)
+				assert.Equal(t, corev1.ConditionFalse, cond.Status)
+				assert.Equal(t, "Fake cluster is running", cond.Message)
+			},
+		},
 	}
 
 	for _, test := range tests {
@@ -370,7 +419,7 @@ func TestReconcile(t *testing.T) {
 				},
 				csrUtil: mockCSRHelper,
 			}
-			_, err := reconciler.Reconcile(reconcile.Request{
+			_, err := reconciler.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{Namespace: namespace, Name: cdName},
 			})
 
@@ -429,7 +478,6 @@ func TestHibernateAfter(t *testing.T) {
 			expectedPowerState: hivev1.HibernatingClusterPowerState,
 		},
 		{
-			name: "cluster due for hibernate older version", // cluster that has never been hibernated and thus has no running condition
 			cd: cdBuilder.Build(
 				testcd.WithHibernateAfter(8*time.Hour),
 				testcd.WithClusterVersion("4.3.11"),
@@ -481,7 +529,9 @@ func TestHibernateAfter(t *testing.T) {
 		{
 			name: "cluster waking from hibernate",
 			setupActuator: func(actuator *mock.MockHibernationActuator) {
-				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(false, nil)
+				actuator.EXPECT().StartMachines(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).Return(nil)
+				actuator.EXPECT().MachinesRunning(gomock.Any(), gomock.Any(), gomock.Any()).Times(1).
+					Return(false, []string{"no-running-1", "no-running-2"}, nil)
 			},
 			cd: cdBuilder.Build(
 				testcd.WithHibernateAfter(8*time.Hour),
@@ -527,6 +577,41 @@ func TestHibernateAfter(t *testing.T) {
 			).Build(),
 			expectedPowerState: hivev1.HibernatingClusterPowerState,
 		},
+		{
+			name: "fake cluster due for hibernate but syncsets not applied",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(8*time.Minute),
+				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 8*time.Minute)),
+				testcd.WithAnnotation(constants.HiveFakeClusterAnnotation, "true"),
+				testcd.InstalledTimestamp(time.Now().Add(-8*time.Minute))),
+			cs: csBuilder.Options(
+				testcs.WithNoFirstSuccessTime(),
+			).Build(),
+			expectError:        true,
+			expectedPowerState: "",
+			expectRequeueAfter: time.Duration(time.Minute * 2),
+		},
+		{
+			name: "fake cluster due for hibernate, syncsets successfully applied",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(8*time.Hour),
+				testcd.WithCondition(hibernatingCondition(corev1.ConditionFalse, hivev1.RunningHibernationReason, 9*time.Hour)),
+				testcd.WithAnnotation(constants.HiveFakeClusterAnnotation, "true"),
+				testcd.InstalledTimestamp(time.Now().Add(-10*time.Hour))),
+			cs: csBuilder.Options(
+				testcs.WithFirstSuccessTime(time.Now()),
+			).Build(),
+			expectedPowerState: hivev1.HibernatingClusterPowerState,
+		},
+		{
+			name: "hibernate fake cluster",
+			cd: cdBuilder.Build(
+				testcd.WithHibernateAfter(1*time.Hour),
+				testcd.InstalledTimestamp(time.Now().Add(-1*time.Hour)),
+				testcd.WithAnnotation(constants.HiveFakeClusterAnnotation, "true")),
+			cs:                 csBuilder.Build(),
+			expectedPowerState: hivev1.HibernatingClusterPowerState,
+		},
 	}
 
 	for _, test := range tests {
@@ -551,7 +636,7 @@ func TestHibernateAfter(t *testing.T) {
 				},
 				csrUtil: mockCSRHelper,
 			}
-			result, err := reconciler.Reconcile(reconcile.Request{
+			result, err := reconciler.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{Namespace: namespace, Name: cdName},
 			})
 
