@@ -1,12 +1,15 @@
 package remotemachineset
 
 import (
+	"bytes"
 	"context"
 	"fmt"
+	"os"
 	"reflect"
 	"strings"
 	"time"
 
+	"github.com/blang/semver/v4"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 
@@ -15,20 +18,25 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/wait"
+	"k8s.io/client-go/util/workqueue"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
+	"sigs.k8s.io/controller-runtime/pkg/event"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
+	"sigs.k8s.io/controller-runtime/pkg/predicate"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
 	autoscalingv1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1"
 	autoscalingv1beta1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1beta1"
 	installertypes "github.com/openshift/installer/pkg/types"
 	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	"github.com/openshift/hive/pkg/awsclient"
 	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
@@ -42,8 +50,18 @@ const (
 	masterMachineLabelSelector = "machine.openshift.io/cluster-api-machine-type=master"
 )
 
-// controllerKind contains the schema.GroupVersionKind for this controller type.
-var controllerKind = hivev1.SchemeGroupVersion.WithKind("MachinePool")
+var (
+	// controllerKind contains the schema.GroupVersionKind for this controller type.
+	controllerKind = hivev1.SchemeGroupVersion.WithKind("MachinePool")
+
+	// remoteMachineSetConditions are the Machine Pool conditions controlled by remote machineset controller
+	remoteMachineSetConditions = []hivev1.MachinePoolConditionType{
+		hivev1.NotEnoughReplicasMachinePoolCondition,
+		hivev1.NoMachinePoolNameLeasesAvailable,
+		hivev1.InvalidSubnetsMachinePoolCondition,
+		hivev1.UnsupportedConfigurationMachinePoolCondition,
+	}
+)
 
 // Add creates a new RemoteMachineSet Controller and adds it to the Manager with default RBAC. The Manager will set fields on the
 // Controller and Start it when the Manager is Started.
@@ -107,9 +125,15 @@ func Add(mgr manager.Manager) error {
 	}
 
 	// Watch for changes to ClusterDeployment
-	err = c.Watch(&source.Kind{Type: &hivev1.ClusterDeployment{}}, &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(r.clusterDeploymentWatchHandler),
-	})
+	err = c.Watch(&source.Kind{Type: &hivev1.ClusterDeployment{}}, handler.EnqueueRequestsFromMapFunc(
+		r.clusterDeploymentWatchHandler,
+	))
+	if err != nil {
+		return err
+	}
+
+	// Periodically watch MachinePools for syncing status from external clusters
+	err = c.Watch(newPeriodicSource(r.Client, 30*time.Minute, r.logger), &handler.EnqueueRequestForObject{})
 	if err != nil {
 		return err
 	}
@@ -117,13 +141,13 @@ func Add(mgr manager.Manager) error {
 	return nil
 }
 
-func (r *ReconcileRemoteMachineSet) clusterDeploymentWatchHandler(a handler.MapObject) []reconcile.Request {
+func (r *ReconcileRemoteMachineSet) clusterDeploymentWatchHandler(a client.Object) []reconcile.Request {
 	retval := []reconcile.Request{}
 
-	cd := a.Object.(*hivev1.ClusterDeployment)
+	cd := a.(*hivev1.ClusterDeployment)
 	if cd == nil {
 		// Wasn't a clusterdeployment, bail out. This should not happen.
-		r.logger.Errorf("Error converting MapObject.Object to ClusterDeployment. Value: %+v", a.Object)
+		r.logger.Errorf("Error converting MapObject.Object to ClusterDeployment. Value: %+v", a)
 		return retval
 	}
 
@@ -131,7 +155,7 @@ func (r *ReconcileRemoteMachineSet) clusterDeploymentWatchHandler(a handler.MapO
 	err := r.List(context.TODO(), pools)
 	if err != nil {
 		// Could not list machine pools
-		r.logger.Errorf("Error listing machine pools. Value: %+v", a.Object)
+		r.logger.Errorf("Error listing machine pools. Value: %+v", a)
 		return retval
 	}
 
@@ -175,19 +199,11 @@ type ReconcileRemoteMachineSet struct {
 
 // Reconcile reads that state of the cluster for a MachinePool object and makes changes to the
 // remote cluster MachineSets based on the state read
-func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	logger := r.logger.WithFields(log.Fields{
-		"machinePool": request.Name,
-		"namespace":   request.Namespace,
-	})
+func (r *ReconcileRemoteMachineSet) Reconcile(ctx context.Context, request reconcile.Request) (reconcile.Result, error) {
+	logger := controllerutils.BuildControllerLogger(ControllerName, "machinePool", request.NamespacedName)
 	logger.Info("reconciling machine pool")
-
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		logger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the MachinePool instance
 	pool := &hivev1.MachinePool{}
@@ -203,6 +219,18 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 		return reconcile.Result{}, err
 	}
 
+	// Initialize machine pool conditions if not present
+	newConditions := controllerutils.InitializeMachinePoolConditions(pool.Status.Conditions, remoteMachineSetConditions)
+	if len(newConditions) > len(pool.Status.Conditions) {
+		pool.Status.Conditions = newConditions
+		logger.Info("initializing remote machineset controller conditions")
+		if err := r.Status().Update(context.TODO(), pool); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to update machine pool status")
+			return reconcile.Result{}, err
+		}
+		return reconcile.Result{}, nil
+	}
+
 	if !controllerutils.HasFinalizer(pool, finalizer) {
 		if pool.DeletionTimestamp != nil {
 			return reconcile.Result{}, nil
@@ -228,7 +256,7 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 		return reconcile.Result{}, err
 	}
 
-	if !controllerutils.ShouldSyncCluster(cd, logger) {
+	if controllerutils.IsClusterPausedOrRelocating(cd, logger) {
 		return reconcile.Result{}, nil
 	}
 
@@ -257,6 +285,11 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 		return reconcile.Result{}, err
 	}
 
+	if controllerutils.IsFakeCluster(cd) {
+		logger.Info("skipping reconcile for fake cluster")
+		return reconcile.Result{}, nil
+	}
+
 	remoteClusterAPIClient, unreachable, requeue := remoteclient.ConnectToRemoteCluster(
 		cd,
 		r.remoteClusterAPIClientBuilder(cd),
@@ -276,18 +309,22 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 
 	remoteMachineSets, err := r.getRemoteMachineSets(remoteClusterAPIClient, logger)
 	if err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not getRemoteMachineSets")
 		return reconcile.Result{}, err
 	}
 
 	generatedMachineSets, proceed, err := r.generateMachineSets(pool, cd, masterMachine, remoteMachineSets, logger)
 	if err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not generateMachineSets")
 		return reconcile.Result{}, err
 	} else if !proceed {
+		logger.Info("machineSets generator indicated not to proceed, returning")
 		return reconcile.Result{}, nil
 	}
 
-	switch result, err := r.ensureEnoughReplicas(pool, generatedMachineSets, logger); {
+	switch result, err := r.ensureEnoughReplicas(pool, generatedMachineSets, cd, logger); {
 	case err != nil:
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not ensureEnoughReplicas")
 		return reconcile.Result{}, err
 	case result != nil:
 		return *result, nil
@@ -295,14 +332,17 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 
 	machineSets, err := r.syncMachineSets(pool, cd, generatedMachineSets, remoteMachineSets, remoteClusterAPIClient, logger)
 	if err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not syncMachineSets")
 		return reconcile.Result{}, err
 	}
 
 	if err := r.syncMachineAutoscalers(pool, cd, machineSets, remoteClusterAPIClient, logger); err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not syncMachineAutoscalers")
 		return reconcile.Result{}, err
 	}
 
 	if err := r.syncClusterAutoscaler(pool, cd, remoteClusterAPIClient, logger); err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "could not syncClusterAutoscaler")
 		return reconcile.Result{}, err
 	}
 
@@ -310,7 +350,7 @@ func (r *ReconcileRemoteMachineSet) Reconcile(request reconcile.Request) (reconc
 		return r.removeFinalizer(pool, logger)
 	}
 
-	return reconcile.Result{}, r.updatePoolStatusForMachineSets(pool, machineSets, logger)
+	return r.updatePoolStatusForMachineSets(pool, machineSets, remoteClusterAPIClient, logger)
 }
 
 func (r *ReconcileRemoteMachineSet) getMasterMachine(
@@ -416,7 +456,7 @@ func (r *ReconcileRemoteMachineSet) generateMachineSets(
 
 // ensureEnoughReplicas ensures that the min replicas in the machine pool is
 // large enough to cover all of the zones for the machine pool. When using
-// auto-scaling, every machineset needs to have a minimum replicas of 1.
+// auto-scaling for some platforms, every machineset needs to have a minimum replicas of 1.
 // If the reconcile.Result returned in non-nil, then the reconciliation loop
 // should stop, returning that result. This is used to prevent re-queueing
 // a machine pool that will always fail with not enough replicas. There is
@@ -425,12 +465,13 @@ func (r *ReconcileRemoteMachineSet) generateMachineSets(
 func (r *ReconcileRemoteMachineSet) ensureEnoughReplicas(
 	pool *hivev1.MachinePool,
 	generatedMachineSets []*machineapi.MachineSet,
+	cd *hivev1.ClusterDeployment,
 	logger log.FieldLogger,
 ) (*reconcile.Result, error) {
 	if pool.Spec.Autoscaling == nil {
 		return nil, nil
 	}
-	if pool.Spec.Autoscaling.MinReplicas < int32(len(generatedMachineSets)) {
+	if pool.Spec.Autoscaling.MinReplicas < int32(len(generatedMachineSets)) && !platformAllowsZeroAutoscalingMinReplicas(cd) {
 		logger.WithField("machinesets", len(generatedMachineSets)).
 			WithField("minReplicas", pool.Spec.Autoscaling.MinReplicas).
 			Warning("when auto-scaling, the MachinePool must have at least one replica for each MachineSet")
@@ -456,7 +497,7 @@ func (r *ReconcileRemoteMachineSet) ensureEnoughReplicas(
 		hivev1.NotEnoughReplicasMachinePoolCondition,
 		corev1.ConditionFalse,
 		"EnoughReplicas",
-		"The MachinePool has at least one replica for each MachineSet",
+		"The MachinePool has sufficient replicas for each MachineSet",
 		controllerutils.UpdateConditionNever,
 	)
 	if changed {
@@ -801,8 +842,9 @@ func (r *ReconcileRemoteMachineSet) syncClusterAutoscaler(
 func (r *ReconcileRemoteMachineSet) updatePoolStatusForMachineSets(
 	pool *hivev1.MachinePool,
 	machineSets []*machineapi.MachineSet,
+	remoteClusterAPIClient client.Client,
 	logger log.FieldLogger,
-) error {
+) (reconcile.Result, error) {
 	origPool := pool.DeepCopy()
 
 	pool.Status.MachineSets = make([]hivev1.MachineSetStatus, len(machineSets))
@@ -815,21 +857,100 @@ func (r *ReconcileRemoteMachineSet) updatePoolStatusForMachineSets(
 		} else {
 			min, max = getMinMaxReplicasForMachineSet(pool, machineSets, i)
 		}
-		pool.Status.MachineSets[i] = hivev1.MachineSetStatus{
-			Name:        ms.Name,
-			Replicas:    *ms.Spec.Replicas,
-			MinReplicas: min,
-			MaxReplicas: max,
+		s := hivev1.MachineSetStatus{
+			Name:          ms.Name,
+			Replicas:      *ms.Spec.Replicas,
+			ReadyReplicas: ms.Status.ReadyReplicas,
+			MinReplicas:   min,
+			MaxReplicas:   max,
+			ErrorReason:   (*string)(ms.Status.ErrorReason),
+			ErrorMessage:  ms.Status.ErrorMessage,
 		}
+		if s.Replicas != s.ReadyReplicas && s.ErrorReason == nil {
+			r, m := summarizeMachinesError(remoteClusterAPIClient, ms, logger)
+			s.ErrorReason = &r
+			s.ErrorMessage = &m
+		}
+
+		pool.Status.MachineSets[i] = s
 		pool.Status.Replicas += *ms.Spec.Replicas
 	}
 
+	var requeueAfter time.Duration
+	for _, ms := range pool.Status.MachineSets {
+		if ms.Replicas != ms.ReadyReplicas {
+			// remote cluster machinesets cannot trigger reconcile and therefore
+			// since we know we are not steady state, we need to ensure that we
+			// requeue to keep the status in sync from remote cluster.
+			requeueAfter = 10 * time.Minute
+			break
+		}
+	}
+
 	if (len(origPool.Status.MachineSets) == 0 && len(pool.Status.MachineSets) == 0) ||
 		reflect.DeepEqual(origPool.Status, pool.Status) {
-		return nil
+		return reconcile.Result{RequeueAfter: requeueAfter}, nil
 	}
 
-	return errors.Wrap(r.Status().Update(context.Background(), pool), "failed to update pool status")
+	return reconcile.Result{RequeueAfter: requeueAfter}, errors.Wrap(r.Status().Update(context.Background(), pool), "failed to update pool status")
+}
+
+// summarizeMachinesError returns reason and message for error state of machineSets by
+// summarizing error reasons and messages from machines the belong to the machineset.
+// If all the machines are in good state, it returns empty reason and message.
+func summarizeMachinesError(remoteClusterAPIClient client.Client, machineSet *machineapi.MachineSet, logger log.FieldLogger) (string, string) {
+	msLog := logger.WithField("machineSet", machineSet.Name)
+
+	sel, err := metav1.LabelSelectorAsSelector(&machineSet.Spec.Selector)
+	if err != nil {
+		msLog.WithError(err).Error("failed to create label selector")
+		return "", ""
+	}
+
+	list := &machineapi.MachineList{}
+	err = remoteClusterAPIClient.List(context.TODO(), list,
+		client.InNamespace(machineSet.GetNamespace()),
+		client.MatchingLabelsSelector{Selector: sel})
+	if err != nil {
+		msLog.WithError(err).Error("failed to list machines for the machineset")
+		return "", ""
+	}
+
+	type errSummary struct {
+		name    string
+		reason  string
+		message string
+	}
+
+	var errs []errSummary
+	for _, m := range list.Items {
+		if m.Status.ErrorReason == nil &&
+			m.Status.ErrorMessage == nil {
+			continue
+		}
+
+		err := errSummary{name: m.GetName()}
+		if m.Status.ErrorReason != nil {
+			err.reason = string(*m.Status.ErrorReason)
+		}
+		if m.Status.ErrorMessage != nil {
+			err.message = *m.Status.ErrorMessage
+		}
+		errs = append(errs, err)
+	}
+
+	if len(errs) == 1 {
+		return errs[0].reason, errs[0].message
+	}
+	if len(errs) > 1 {
+		r := "MultipleMachinesFailed"
+		m := &bytes.Buffer{}
+		for _, e := range errs {
+			fmt.Fprintf(m, "Machine %s failed (%s): %s,\n", e.name, e.reason, e.message)
+		}
+		return r, m.String()
+	}
+	return "", ""
 }
 
 func (r *ReconcileRemoteMachineSet) createActuator(
@@ -841,16 +962,18 @@ func (r *ReconcileRemoteMachineSet) createActuator(
 ) (Actuator, error) {
 	switch {
 	case cd.Spec.Platform.AWS != nil:
-		creds := &corev1.Secret{}
-		if err := r.Get(
-			context.TODO(),
-			types.NamespacedName{
-				Name:      cd.Spec.Platform.AWS.CredentialsSecretRef.Name,
+		creds := awsclient.CredentialsSource{
+			Secret: &awsclient.SecretCredentialsSource{
+				Ref:       &cd.Spec.Platform.AWS.CredentialsSecretRef,
 				Namespace: cd.Namespace,
 			},
-			creds,
-		); err != nil {
-			return nil, err
+			AssumeRole: &awsclient.AssumeRoleCredentialsSource{
+				SecretRef: corev1.SecretReference{
+					Namespace: controllerutils.GetHiveNamespace(),
+					Name:      os.Getenv(constants.HiveAWSServiceProviderCredentialsSecretRefEnvVar),
+				},
+				Role: cd.Spec.Platform.AWS.CredentialsAssumeRole,
+			},
 		}
 		return NewAWSActuator(r.Client, creds, cd.Spec.Platform.AWS.Region, pool, masterMachine, r.scheme, logger)
 	case cd.Spec.Platform.GCP != nil:
@@ -884,7 +1007,7 @@ func (r *ReconcileRemoteMachineSet) createActuator(
 		}
 		return NewAzureActuator(creds, logger)
 	case cd.Spec.Platform.OpenStack != nil:
-		return NewOpenStackActuator(masterMachine, r.scheme, logger)
+		return NewOpenStackActuator(masterMachine, r.scheme, r.Client, logger)
 	case cd.Spec.Platform.VSphere != nil:
 		return NewVSphereActuator(masterMachine, r.scheme, logger)
 	case cd.Spec.Platform.Ovirt != nil:
@@ -942,3 +1065,103 @@ func getClusterVersion(cd *hivev1.ClusterDeployment) (string, error) {
 	}
 	return version, nil
 }
+
+func platformAllowsZeroAutoscalingMinReplicas(cd *hivev1.ClusterDeployment) bool {
+	// Since 4.5, AWS, Azure, and GCP allow zero-sized minReplicas for autoscaling
+	if cd.Spec.Platform.AWS != nil || cd.Spec.Platform.Azure != nil || cd.Spec.Platform.GCP != nil {
+		return true
+	}
+
+	// Since 4.7, OpenStack allows zero-sized minReplicas for autoscaling
+	if cd.Spec.Platform.OpenStack != nil {
+		majorMinorPatch, ok := cd.Labels[constants.VersionMajorMinorPatchLabel]
+		if !ok {
+			// can't determine whether to allow zero minReplicas
+			return false
+		}
+
+		currentVersion, err := semver.Make(majorMinorPatch)
+		if err != nil {
+			// assume we can't set minReplicas to zero
+			return false
+		}
+
+		minimumOpenStackVersion, err := semver.Make("4.7.0")
+		if err != nil {
+			// something terrible has happened
+			return false
+		}
+
+		if currentVersion.GTE(minimumOpenStackVersion) {
+			return true
+		}
+
+		return false
+	}
+
+	return false
+}
+
+// periodicSource uses the client to list the machinepools
+// every duration (including some jitter) and creates a Generic
+// event for each object.
+// this is useful to create a steady stream of reconcile requests
+// when some of the changes cannot be models in Watches.
+type periodicSource struct {
+	client   client.Client
+	duration time.Duration
+
+	logger log.FieldLogger
+}
+
+func newPeriodicSource(c client.Client, d time.Duration, logger log.FieldLogger) *periodicSource {
+	return &periodicSource{
+		client:   c,
+		duration: d,
+		logger:   logger,
+	}
+}
+
+// Start is internal and should be called only by the Controller to register an EventHandler with the Informer
+// to enqueue reconcile.Requests.
+func (ps *periodicSource) Start(ctx context.Context,
+	handler handler.EventHandler,
+	queue workqueue.RateLimitingInterface,
+	prcts ...predicate.Predicate) error {
+
+	go wait.JitterUntilWithContext(ctx, ps.syncFunc(handler, queue, prcts...), ps.duration, 0.1, true)
+	return nil
+}
+
+func (ps *periodicSource) syncFunc(handler handler.EventHandler,
+	queue workqueue.RateLimitingInterface,
+	prcts ...predicate.Predicate) func(context.Context) {
+
+	return func(ctx context.Context) {
+		mpList := &hivev1.MachinePoolList{}
+		err := ps.client.List(ctx, mpList)
+		if err != nil {
+			ps.logger.WithError(err).Error("failed to list MachinePools")
+			return
+		}
+
+		for idx := range mpList.Items {
+			evt := event.GenericEvent{Object: &mpList.Items[idx]}
+
+			shouldHandle := true
+			for _, p := range prcts {
+				if !p.Generic(evt) {
+					shouldHandle = false
+				}
+			}
+
+			if shouldHandle {
+				handler.Generic(evt, queue)
+			}
+		}
+	}
+}
+
+func (ps *periodicSource) String() string {
+	return fmt.Sprintf("periodic source for MachinePools every %s", ps.duration)
+}
