@@ -11,16 +11,14 @@ import (
 
 	log "github.com/sirupsen/logrus"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
 	"github.com/openshift/hive/pkg/resource"
 
 	"github.com/openshift/library-go/pkg/operator/events"
 
-	apiextclientv1beta1 "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/typed/apiextensions/v1beta1"
-	apiregclientv1 "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset/typed/apiregistration/v1"
-
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
+	apiextv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -43,11 +41,14 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/predicate"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
+
+	"github.com/openshift/hive/pkg/constants"
+	"github.com/openshift/hive/pkg/operator/metrics"
+	"github.com/openshift/hive/pkg/operator/util"
 )
 
 const (
-	// hiveConfigName is the one and only name for a HiveConfig supported in the cluster. Any others will be ignored.
-	hiveConfigName = "hive"
+	ControllerName = hivev1.HiveControllerName
 
 	hiveOperatorDeploymentName = "hive-operator"
 
@@ -61,6 +62,14 @@ const (
 	watchResyncInterval = 30 * time.Minute
 )
 
+var (
+	// HiveConfigConditions are the HiveConfig conditions controlled by
+	// hive controller
+	HiveConfigConditions = []hivev1.HiveConfigConditionType{
+		hivev1.HiveReadyCondition,
+	}
+)
+
 // Add creates a new Hive Controller and adds it to the Manager with default RBAC. The Manager will set fields on the Controller
 // and Start it when the Manager is Started.
 func Add(mgr manager.Manager) error {
@@ -69,7 +78,12 @@ func Add(mgr manager.Manager) error {
 
 // newReconciler returns a new reconcile.Reconciler
 func newReconciler(mgr manager.Manager) reconcile.Reconciler {
-	return &ReconcileHiveConfig{Client: mgr.GetClient(), scheme: mgr.GetScheme(), restConfig: mgr.GetConfig(), mgr: mgr}
+	return &ReconcileHiveConfig{
+		Client:     mgr.GetClient(),
+		scheme:     mgr.GetScheme(),
+		restConfig: mgr.GetConfig(),
+		mgr:        mgr,
+	}
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
@@ -89,16 +103,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	r.(*ReconcileHiveConfig).apiregClient, err = apiregclientv1.NewForConfig(mgr.GetConfig())
-	if err != nil {
-		return err
-	}
-
-	r.(*ReconcileHiveConfig).apiextClient, err = apiextclientv1beta1.NewForConfig(mgr.GetConfig())
-	if err != nil {
-		return err
-	}
-
 	r.(*ReconcileHiveConfig).discoveryClient, err = discovery.NewDiscoveryClientForConfig(mgr.GetConfig())
 	if err != nil {
 		return err
@@ -137,7 +141,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		mgr.Add(&informerRunnable{informer: configMapInformer})
 
 		// Watch for changes to cm/kube-apiserver-aggregator-client-ca in the OpenShift managed namespace
-		err = c.Watch(&source.Informer{Informer: configMapInformer}, &handler.EnqueueRequestsFromMapFunc{ToRequests: handler.ToRequestsFunc(aggregatorCAConfigMapHandler)})
+		err = c.Watch(&source.Informer{Informer: configMapInformer}, handler.EnqueueRequestsFromMapFunc(handler.MapFunc(aggregatorCAConfigMapHandler)))
 		if err != nil {
 			return err
 		}
@@ -153,10 +157,25 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
+	err = mgr.GetFieldIndexer().IndexField(context.TODO(), &hivev1.HiveConfig{}, "spec.secrets.secretName", func(o client.Object) []string {
+		var res []string
+		instance := o.(*hivev1.HiveConfig)
+
+		// add all the secret objects to res that should trigger resync of HiveConfig
+
+		for _, lObj := range instance.Spec.AdditionalCertificateAuthoritiesSecretRef {
+			res = append(res, lObj.Name)
+		}
+
+		return res
+	})
+	if err != nil {
+		return err
+	}
+
 	// Monitor changes to DaemonSets:
 	err = c.Watch(&source.Kind{Type: &appsv1.DaemonSet{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &hivev1.HiveConfig{},
+		OwnerType: &hivev1.HiveConfig{},
 	})
 	if err != nil {
 		return err
@@ -164,8 +183,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 
 	// Monitor changes to Deployments:
 	err = c.Watch(&source.Kind{Type: &appsv1.Deployment{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &hivev1.HiveConfig{},
+		OwnerType: &hivev1.HiveConfig{},
 	})
 	if err != nil {
 		return err
@@ -173,13 +191,44 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 
 	// Monitor changes to Services:
 	err = c.Watch(&source.Kind{Type: &corev1.Service{}}, &handler.EnqueueRequestForOwner{
-		IsController: true,
-		OwnerType:    &hivev1.HiveConfig{},
+		OwnerType: &hivev1.HiveConfig{},
 	})
 	if err != nil {
 		return err
 	}
 
+	// Monitor changes to StatefulSets:
+	err = c.Watch(&source.Kind{Type: &appsv1.StatefulSet{}}, &handler.EnqueueRequestForOwner{
+		OwnerType: &hivev1.HiveConfig{},
+	})
+	if err != nil {
+		return err
+	}
+
+	// Monitor CRDs so that we can keep latest list of supported contracts
+	err = c.Watch(&source.Kind{Type: &apiextv1beta1.CustomResourceDefinition{}},
+		handler.EnqueueRequestsFromMapFunc(func(_ client.Object) []reconcile.Request {
+			retval := []reconcile.Request{}
+
+			configList := &hivev1.HiveConfigList{}
+			err := r.(*ReconcileHiveConfig).List(context.TODO(), configList) // reconcile all HiveConfigs
+			if err != nil {
+				log.WithError(err).Errorf("error listing hive configs for CRD reconcile")
+				return retval
+			}
+
+			for _, config := range configList.Items {
+				retval = append(retval, reconcile.Request{NamespacedName: types.NamespacedName{
+					Name: config.Name,
+				}})
+			}
+			log.WithField("configs", retval).Debug("reconciled for change in CRD")
+			return retval
+		}))
+	if err != nil {
+		return err
+	}
+
 	// Lookup the hive-operator Deployment image, we will assume hive components should all be
 	// using the same image as the operator.
 	operatorDeployment := &appsv1.Deployment{}
@@ -211,35 +260,36 @@ var _ reconcile.Reconciler = &ReconcileHiveConfig{}
 // ReconcileHiveConfig reconciles a Hive object
 type ReconcileHiveConfig struct {
 	client.Client
-	scheme                            *runtime.Scheme
-	kubeClient                        kubernetes.Interface
-	apiextClient                      *apiextclientv1beta1.ApiextensionsV1beta1Client
-	apiregClient                      *apiregclientv1.ApiregistrationV1Client
-	discoveryClient                   discovery.DiscoveryInterface
-	dynamicClient                     dynamic.Interface
-	restConfig                        *rest.Config
-	hiveImage                         string
-	hiveOperatorNamespace             string
-	hiveImagePullPolicy               corev1.PullPolicy
-	syncAggregatorCA                  bool
-	managedConfigCMLister             corev1listers.ConfigMapLister
-	ctrlr                             controller.Controller
-	servingCertSecretWatchEstablished bool
-	mgr                               manager.Manager
+	scheme                 *runtime.Scheme
+	kubeClient             kubernetes.Interface
+	discoveryClient        discovery.DiscoveryInterface
+	dynamicClient          dynamic.Interface
+	restConfig             *rest.Config
+	hiveImage              string
+	hiveOperatorNamespace  string
+	hiveImagePullPolicy    corev1.PullPolicy
+	syncAggregatorCA       bool
+	managedConfigCMLister  corev1listers.ConfigMapLister
+	ctrlr                  controller.Controller
+	hiveSecretLister       corev1listers.SecretLister
+	secretWatchEstablished bool
+	mgr                    manager.Manager
 }
 
 // Reconcile reads that state of the cluster for a Hive object and makes changes based on the state read
 // and what is in the Hive.Spec
-func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Result, error) {
+func (r *ReconcileHiveConfig) Reconcile(ctx context.Context, request reconcile.Request) (reconcile.Result, error) {
 	hLog := log.WithField("controller", "hive")
 	hLog.Info("Reconciling Hive components")
+	recobsrv := metrics.NewReconcileObserver(ControllerName, hLog)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the Hive instance
 	instance := &hivev1.HiveConfig{}
 
 	// We only support one HiveConfig per cluster, and it must be called "hive". This prevents installing
 	// Hive more than once in the cluster.
-	if request.NamespacedName.Name != hiveConfigName {
+	if request.NamespacedName.Name != constants.HiveConfigName {
 		hLog.WithField("hiveConfig", request.NamespacedName.Name).Warn(
 			"invalid HiveConfig name, only one HiveConfig supported per cluster and must be named 'hive'")
 		return reconcile.Result{}, nil
@@ -253,7 +303,7 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 			// Object not found, return.  Created objects are automatically garbage collected.
 			// For additional cleanup logic use finalizers.
 			hLog.Debug("HiveConfig not found, deleted?")
-			r.servingCertSecretWatchEstablished = false
+			r.secretWatchEstablished = false
 			return reconcile.Result{}, nil
 		}
 		// Error reading the object - requeue the request.
@@ -264,7 +314,18 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 	origHiveConfig := instance.DeepCopy()
 	hiveNSName := getHiveNamespace(instance)
 
+	// Initialize HiveConfig conditions if not present
+	newConditions := util.InitializeHiveConfigConditions(instance.Status.Conditions, HiveConfigConditions)
+	if len(newConditions) > len(origHiveConfig.Status.Conditions) {
+		instance.Status.Conditions = newConditions
+		hLog.Info("initializing hive controller conditions")
+		err = r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
 	if err := r.establishSecretWatch(hLog, hiveNSName); err != nil {
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorEstablishingSecretWatch", err.Error())
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
 
@@ -284,6 +345,8 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 			hLog.WithField("hiveNS", hiveNSName).Debug("target namespace already exists")
 		} else {
 			hLog.WithError(err).Error("error creating hive target namespace")
+			instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorCreatingHiveNamespace", err.Error())
+			r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 			return reconcile.Result{}, err
 		}
 	} else {
@@ -323,6 +386,7 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 	h, err := resource.NewHelperFromRESTConfig(r.restConfig, hLog)
 	if err != nil {
 		hLog.WithError(err).Error("error creating resource helper")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorCreatingResourceHelper", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
@@ -330,13 +394,38 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 	managedDomainsConfigMap, err := r.configureManagedDomains(hLog, instance)
 	if err != nil {
 		hLog.WithError(err).Error("error setting up managed domains")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorSettingUpManagedDomains", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
 
-	confighash, err := r.deployHiveControllersConfigMap(hLog, h, instance)
+	plConfigHash, err := r.deployAWSPrivateLinkConfigMap(hLog, h, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error deploying aws privatelink configmap")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingAWSPrivatelinkConfigmap", err.Error())
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
+	scConfigHash, err := r.deploySupportedContractsConfigMap(hLog, h, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error deploying supported contracts configmap")
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
+	confighash, err := r.deployHiveControllersConfigMap(hLog, h, instance, plConfigHash)
 	if err != nil {
 		hLog.WithError(err).Error("error deploying controllers configmap")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingControllersConfigmap", err.Error())
+		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
+		return reconcile.Result{}, err
+	}
+
+	fgConfigHash, err := r.deployFeatureGatesConfigMap(hLog, h, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error deploying feature gates configmap")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingFeatureGatesConfigmap", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
@@ -344,6 +433,7 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 	err = r.deployHive(hLog, h, instance, recorder, managedDomainsConfigMap, confighash)
 	if err != nil {
 		hLog.WithError(err).Error("error deploying Hive")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingHive", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
@@ -351,6 +441,7 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 	err = r.deployClusterSync(hLog, h, instance, confighash)
 	if err != nil {
 		hLog.WithError(err).Error("error deploying ClusterSync")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingClusterSync", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
@@ -360,19 +451,22 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 		return reconcile.Result{}, err
 	}
 
-	err = r.deployHiveAdmission(hLog, h, instance, recorder, managedDomainsConfigMap)
+	err = r.deployHiveAdmission(hLog, h, instance, recorder, managedDomainsConfigMap, fgConfigHash, plConfigHash, scConfigHash)
 	if err != nil {
 		hLog.WithError(err).Error("error deploying HiveAdmission")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeployingHiveAdmission", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
 
 	if err := r.cleanupLegacySyncSetInstances(hLog); err != nil {
 		hLog.WithError(err).Error("error cleaning up legacy SyncSetInstances")
+		instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionFalse, "ErrorDeletingLegacySyncSetInstances", err.Error())
 		r.updateHiveConfigStatus(origHiveConfig, instance, hLog, false)
 		return reconcile.Result{}, err
 	}
 
+	instance.Status.Conditions = util.SetHiveConfigCondition(instance.Status.Conditions, hivev1.HiveReadyCondition, corev1.ConditionTrue, "DeploymentSuccess", "Hive is deployed successfully")
 	if err := r.updateHiveConfigStatus(origHiveConfig, instance, hLog, true); err != nil {
 		return reconcile.Result{}, err
 	}
@@ -383,13 +477,14 @@ func (r *ReconcileHiveConfig) Reconcile(request reconcile.Request) (reconcile.Re
 func (r *ReconcileHiveConfig) establishSecretWatch(hLog *log.Entry, hiveNSName string) error {
 	// We need to establish a watch on Secret in the Hive namespace, one time only. We do not know this namespace until
 	// we have a HiveConfig.
-	if !r.servingCertSecretWatchEstablished {
+	if !r.secretWatchEstablished {
 		hLog.WithField("namespace", hiveNSName).Info("establishing watch on secrets in hive namespace")
 
 		// Create an informer that only listens to events in the OpenShift managed namespace
 		kubeInformerFactory := kubeinformers.NewSharedInformerFactoryWithOptions(r.kubeClient, watchResyncInterval,
 			kubeinformers.WithNamespace(hiveNSName))
 		secretsInformer := kubeInformerFactory.Core().V1().Secrets().Informer()
+		r.hiveSecretLister = kubeInformerFactory.Core().V1().Secrets().Lister()
 		if err := r.mgr.Add(&informerRunnable{informer: secretsInformer}); err != nil {
 			hLog.WithError(err).Error("error adding secret informer to manager")
 			return err
@@ -400,27 +495,56 @@ func (r *ReconcileHiveConfig) establishSecretWatch(hLog *log.Entry, hiveNSName s
 		err := r.ctrlr.Watch(&source.Informer{Informer: secretsInformer}, handler.Funcs{
 			CreateFunc: func(e event.CreateEvent, q workqueue.RateLimitingInterface) {
 				hLog.Debug("eventHandler CreateFunc")
-				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{Name: hiveConfigName}})
+				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{Name: constants.HiveConfigName}})
 			},
 			UpdateFunc: func(e event.UpdateEvent, q workqueue.RateLimitingInterface) {
 				hLog.Debug("eventHandler UpdateFunc")
-				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{Name: hiveConfigName}})
+				q.Add(reconcile.Request{NamespacedName: types.NamespacedName{Name: constants.HiveConfigName}})
 			},
 		}, predicate.Funcs{
 			CreateFunc: func(e event.CreateEvent) bool {
-				hLog.WithField("predicateResponse", e.Meta.GetName() == hiveAdmissionServingCertSecretName).Debug("secret CreateEvent")
-				return e.Meta.GetName() == hiveAdmissionServingCertSecretName
+				hLog.WithField("predicateResponse", e.Object.GetName() == hiveAdmissionServingCertSecretName).Debug("secret CreateEvent")
+				return e.Object.GetName() == hiveAdmissionServingCertSecretName
 			},
 			UpdateFunc: func(e event.UpdateEvent) bool {
-				hLog.WithField("predicateResponse", e.MetaNew.GetName() == hiveAdmissionServingCertSecretName).Debug("secret UpdateEvent")
-				return e.MetaNew.GetName() == hiveAdmissionServingCertSecretName
+				hLog.WithField("predicateResponse", e.ObjectNew.GetName() == hiveAdmissionServingCertSecretName).Debug("secret UpdateEvent")
+				return e.ObjectNew.GetName() == hiveAdmissionServingCertSecretName
 			},
 		})
 		if err != nil {
 			hLog.WithError(err).Error("error establishing secret watch")
 			return err
 		}
-		r.servingCertSecretWatchEstablished = true
+
+		// Watch secrets in HiveConfig that should trigger reconcile on change.
+		err = r.ctrlr.Watch(&source.Informer{Informer: secretsInformer}, handler.EnqueueRequestsFromMapFunc(func(a client.Object) []reconcile.Request {
+			retval := []reconcile.Request{}
+			secret, ok := a.(*corev1.Secret)
+			if !ok {
+				// Wasn't a Secret, bail out. This should not happen.
+				hLog.Errorf("Error converting MapObject.Object to Secret. Value: %+v", a)
+				return retval
+			}
+
+			configWithSecrets := &hivev1.HiveConfigList{}
+			err := r.mgr.GetClient().List(context.Background(), configWithSecrets, client.MatchingFields{"spec.secrets.secretName": secret.Name})
+			if err != nil {
+				hLog.Errorf("Error listing HiveConfigs for secret %s: %v", secret.Name, err)
+				return retval
+			}
+			for _, config := range configWithSecrets.Items {
+				retval = append(retval, reconcile.Request{NamespacedName: types.NamespacedName{
+					Name: config.Name,
+				}})
+			}
+			hLog.WithField("secretName", secret.Name).WithField("configs", retval).Debug("secret change trigger reconcile for HiveConfigs")
+			return retval
+		}))
+		if err != nil {
+			return err
+		}
+
+		r.secretWatchEstablished = true
 	} else {
 		hLog.Debug("secret watch already established")
 	}
@@ -445,15 +569,16 @@ type informerRunnable struct {
 	informer cache.SharedIndexInformer
 }
 
-func (r *informerRunnable) Start(stopch <-chan struct{}) error {
+func (r *informerRunnable) Start(ctx context.Context) error {
+	stopch := ctx.Done()
 	r.informer.Run(stopch)
 	cache.WaitForCacheSync(stopch, r.informer.HasSynced)
 	return nil
 }
 
-func aggregatorCAConfigMapHandler(o handler.MapObject) []reconcile.Request {
-	if o.Meta.GetName() == aggregatorCAConfigMapName {
-		return []reconcile.Request{{NamespacedName: types.NamespacedName{Name: hiveConfigName}}}
+func aggregatorCAConfigMapHandler(o client.Object) []reconcile.Request {
+	if o.GetName() == aggregatorCAConfigMapName {
+		return []reconcile.Request{{NamespacedName: types.NamespacedName{Name: constants.HiveConfigName}}}
 	}
 	return nil
 }
