@@ -4,15 +4,23 @@ import (
 	"context"
 	"crypto/md5"
 	"encoding/hex"
+	"encoding/json"
 	"fmt"
+	"os"
+	"sort"
+	"strings"
 
 	log "github.com/sirupsen/logrus"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hivecontractsv1alpha1 "github.com/openshift/hive/apis/hivecontracts/v1alpha1"
+	"github.com/openshift/hive/pkg/constants"
+	hiveconstants "github.com/openshift/hive/pkg/constants"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/operator/assets"
 	"github.com/openshift/hive/pkg/operator/util"
 	"github.com/openshift/hive/pkg/resource"
+	"github.com/openshift/hive/pkg/util/contracts"
 
 	"github.com/openshift/library-go/pkg/operator/events"
 	"github.com/openshift/library-go/pkg/operator/resource/resourceread"
@@ -22,6 +30,7 @@ import (
 	apiextv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/client-go/kubernetes/scheme"
 	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	"sigs.k8s.io/controller-runtime/pkg/client"
@@ -37,6 +46,17 @@ const (
 	servingCertSecretHashAnnotation  = "hive.openshift.io/serving-cert-secret-hash"
 )
 
+const (
+	featureGateConfigMapName = "hive-feature-gates"
+	inputHashAnnotation      = "hive.openshift.io/hive-admission-input-sources-hash"
+)
+
+const (
+	supportedContractsConfigMapName      = "hive-supported-contracts"
+	supportedContractsConfigMapNameKey   = "supported-contracts"
+	supportedContractsConfigMapMountPath = "/data/supported-contracts-config"
+)
+
 var webhookAssets = []string{
 	"config/hiveadmission/clusterdeployment-webhook.yaml",
 	"config/hiveadmission/clusterimageset-webhook.yaml",
@@ -47,7 +67,7 @@ var webhookAssets = []string{
 	"config/hiveadmission/selectorsyncset-webhook.yaml",
 }
 
-func (r *ReconcileHiveConfig) deployHiveAdmission(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig, recorder events.Recorder, mdConfigMap *corev1.ConfigMap) error {
+func (r *ReconcileHiveConfig) deployHiveAdmission(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig, recorder events.Recorder, mdConfigMap *corev1.ConfigMap, additionalHashes ...string) error {
 	hiveNSName := getHiveNamespace(instance)
 
 	// Load namespaced assets, decode them, set to our target namespace, and apply:
@@ -104,7 +124,18 @@ func (r *ReconcileHiveConfig) deployHiveAdmission(hLog log.FieldLogger, h resour
 	hiveAdmDeployment.Annotations[aggregatorClientCAHashAnnotation] = instance.Status.AggregatorClientCAHash
 	hiveAdmDeployment.Spec.Template.ObjectMeta.Annotations[aggregatorClientCAHashAnnotation] = instance.Status.AggregatorClientCAHash
 
+	hasher := md5.New()
+	for _, v := range additionalHashes {
+		hasher.Write([]byte(v))
+	}
+	hiveAdmDeployment.Spec.Template.ObjectMeta.Annotations[inputHashAnnotation] = hex.EncodeToString(hasher.Sum(nil))
+	controllerutils.SetProxyEnvVars(&hiveAdmDeployment.Spec.Template.Spec,
+		os.Getenv("HTTP_PROXY"), os.Getenv("HTTPS_PROXY"), os.Getenv("NO_PROXY"))
+
 	addManagedDomainsVolume(&hiveAdmDeployment.Spec.Template.Spec, mdConfigMap.Name)
+	addAWSPrivateLinkConfigVolume(&hiveAdmDeployment.Spec.Template.Spec)
+	addSupportedContractsConfigVolume(&hiveAdmDeployment.Spec.Template.Spec)
+	addReleaseImageVerificationConfigMapEnv(&hiveAdmDeployment.Spec.Template.Spec, instance)
 
 	validatingWebhooks := make([]*admregv1.ValidatingWebhookConfiguration, len(webhookAssets))
 	for i, yaml := range webhookAssets {
@@ -143,10 +174,11 @@ func (r *ReconcileHiveConfig) deployHiveAdmission(hLog log.FieldLogger, h resour
 	}
 
 	// Set the serving cert CA secret hash as an annotation on the pod template to force a rollout in the event it changes:
-	servingCertSecret := &corev1.Secret{}
-	if err := r.Client.Get(context.Background(), types.NamespacedName{Namespace: hiveNSName, Name: hiveAdmissionServingCertSecretName}, servingCertSecret); err != nil {
+	servingCertSecret, err := r.hiveSecretLister.Secrets(hiveNSName).Get(hiveAdmissionServingCertSecretName)
+	if err != nil {
 		hLog.WithError(err).WithField("secretName", hiveAdmissionServingCertSecretName).Log(
 			controllerutils.LogLevel(err), "error getting serving cert secret")
+		return err
 	}
 	hLog.Info("Hashing serving cert secret onto a hiveadmission deployment annotation")
 	certSecretHash := computeSecretDataHash(servingCertSecret.Data)
@@ -266,3 +298,161 @@ func (r *ReconcileHiveConfig) is311(hLog log.FieldLogger) (bool, error) {
 	}
 	return false, nil
 }
+
+func (r *ReconcileHiveConfig) deployFeatureGatesConfigMap(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig) (string, error) {
+	cm := &corev1.ConfigMap{}
+	cm.Name = featureGateConfigMapName
+	cm.Namespace = getHiveNamespace(instance)
+	cm.Data = make(map[string]string)
+
+	cm.Data[constants.HiveFeatureGatesEnabledEnvVar] = ""
+	if fg := instance.Spec.FeatureGates; fg != nil {
+		s, ok := hivev1.FeatureSets[fg.FeatureSet]
+		if ok && s != nil {
+			cm.Data[constants.HiveFeatureGatesEnabledEnvVar] = strings.Join(s.Enabled, ",")
+		}
+		if fg.FeatureSet == hivev1.CustomFeatureSet && fg.Custom != nil {
+			cm.Data[constants.HiveFeatureGatesEnabledEnvVar] = strings.Join(fg.Custom.Enabled, ",")
+		}
+	}
+
+	result, err := util.ApplyRuntimeObjectWithGC(h, cm, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error applying hive-feature-gates configmap")
+		return "", err
+	}
+	hLog.WithField("result", result).Info("hive-feature-gates configmap applied")
+
+	return computeConfigHash(cm), nil
+}
+
+// allowedContracts is the list of operator whitelisted contracts that hive will accept
+// from CRDs.
+var allowedContracts = sets.NewString(
+	hivecontractsv1alpha1.ClusterInstallContractLabelKey,
+)
+
+// knowContracts is a list of contracts and their implementations that doesn't
+// require discovery using CRDs
+var knowContracts = contracts.SupportedContractImplementationsList{{
+	Name: hivecontractsv1alpha1.ClusterInstallContractName,
+	Supported: []contracts.ContractImplementation{{
+		Group:   "extensions.hive.openshift.io",
+		Version: "v1beta1",
+		Kind:    "AgentClusterInstall",
+	}},
+}}
+
+func (r *ReconcileHiveConfig) deploySupportedContractsConfigMap(hLog log.FieldLogger, h resource.Helper, instance *hivev1.HiveConfig) (string, error) {
+	cm := &corev1.ConfigMap{}
+	cm.Name = supportedContractsConfigMapName
+	cm.Namespace = getHiveNamespace(instance)
+	cm.Data = make(map[string]string)
+
+	supported := map[string][]contracts.ContractImplementation{}
+	for _, k := range knowContracts {
+		supported[k.Name] = k.Supported
+	}
+
+	crdList := &apiextv1beta1.CustomResourceDefinitionList{}
+	if err := r.Client.List(context.TODO(), crdList); err != nil {
+		hLog.WithError(err).Error("error getting crds for collect contract implementations")
+		return "", err
+	}
+	for _, crd := range crdList.Items {
+		// collect all the possible implementations from this crd
+		var impls []contracts.ContractImplementation
+		for _, version := range crd.Spec.Versions {
+			impls = append(impls, contracts.ContractImplementation{
+				Group:   crd.Spec.Group,
+				Version: version.Name,
+				Kind:    crd.Spec.Names.Kind,
+			})
+		}
+
+		for label, val := range crd.ObjectMeta.Labels {
+			if strings.HasPrefix(label, "contracts.hive.openshift.io") &&
+				val == "true" &&
+				allowedContracts.Has(label) { // lets store impls for this contract
+				contract := strings.TrimPrefix(label, "contracts.hive.openshift.io/")
+				curr := supported[contract]
+				curr = append(curr, impls...)
+				supported[contract] = curr
+			}
+		}
+	}
+
+	if len(supported) > 0 {
+		var keys []string
+		for k := range supported {
+			keys = append(keys, k)
+		}
+		sort.Strings(keys)
+
+		var config contracts.SupportedContractImplementationsList
+		for _, c := range keys {
+			config = append(config, contracts.SupportedContractImplementations{
+				Name:      c,
+				Supported: supported[c],
+			})
+		}
+
+		configRaw, err := json.Marshal(config)
+		if err != nil {
+			hLog.WithError(err).Error("failed to marshal the supported contracts config")
+			return "", err
+		}
+		cm.Data[supportedContractsConfigMapNameKey] = string(configRaw)
+	}
+
+	result, err := util.ApplyRuntimeObjectWithGC(h, cm, instance)
+	if err != nil {
+		hLog.WithError(err).Error("error applying hive-supported-contracts configmap")
+		return "", err
+	}
+	hLog.WithField("result", result).Info("hive-supported-contracts configmap applied")
+
+	return computeConfigHash(cm), nil
+}
+
+func addSupportedContractsConfigVolume(podSpec *corev1.PodSpec) {
+	optional := true
+	volume := corev1.Volume{}
+	volume.Name = supportedContractsConfigMapName
+	volume.ConfigMap = &corev1.ConfigMapVolumeSource{
+		LocalObjectReference: corev1.LocalObjectReference{
+			Name: supportedContractsConfigMapName,
+		},
+		Optional: &optional,
+	}
+	volumeMount := corev1.VolumeMount{
+		Name:      supportedContractsConfigMapName,
+		MountPath: supportedContractsConfigMapMountPath,
+	}
+	envVar := corev1.EnvVar{
+		Name:  constants.SupportedContractImplementationsFileEnvVar,
+		Value: fmt.Sprintf("%s/%s", supportedContractsConfigMapMountPath, supportedContractsConfigMapNameKey),
+	}
+	podSpec.Volumes = append(podSpec.Volumes, volume)
+	podSpec.Containers[0].VolumeMounts = append(podSpec.Containers[0].VolumeMounts, volumeMount)
+	podSpec.Containers[0].Env = append(podSpec.Containers[0].Env, envVar)
+}
+
+func addReleaseImageVerificationConfigMapEnv(podSpec *corev1.PodSpec, instance *hivev1.HiveConfig) {
+	if instance.Spec.ReleaseImageVerificationConfigMapRef == nil {
+		return
+	}
+	podSpec.Containers[0].Env = append(podSpec.Containers[0].Env, corev1.EnvVar{
+		Name:  hiveconstants.HiveReleaseImageVerificationConfigMapNamespaceEnvVar,
+		Value: instance.Spec.ReleaseImageVerificationConfigMapRef.Namespace,
+	}, corev1.EnvVar{
+		Name:  hiveconstants.HiveReleaseImageVerificationConfigMapNameEnvVar,
+		Value: instance.Spec.ReleaseImageVerificationConfigMapRef.Name,
+	})
+}
+
+func computeConfigHash(cm *corev1.ConfigMap) string {
+	hasher := md5.New()
+	hasher.Write([]byte(fmt.Sprintf("%v", cm.Data)))
+	return hex.EncodeToString(hasher.Sum(nil))
+}
