@@ -6,7 +6,10 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hivev1aws "github.com/openshift/hive/apis/hive/v1/aws"
+	hivev1azure "github.com/openshift/hive/apis/hive/v1/azure"
+	hivev1gcp "github.com/openshift/hive/apis/hive/v1/gcp"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/test/generic"
 )
@@ -90,6 +93,11 @@ func WithLabel(key, value string) Option {
 	return Generic(generic.WithLabel(key, value))
 }
 
+// WithAnnotation sets the specified annotation on the supplied object.
+func WithAnnotation(key, value string) Option {
+	return Generic(generic.WithAnnotation(key, value))
+}
+
 // WithCondition adds the specified condition to the ClusterDeployment
 func WithCondition(cond hivev1.ClusterDeploymentCondition) Option {
 	return func(clusterDeployment *hivev1.ClusterDeployment) {
@@ -130,6 +138,12 @@ func InstalledTimestamp(instTime time.Time) Option {
 	}
 }
 
+func InstallRestarts(restarts int) Option {
+	return func(clusterDeployment *hivev1.ClusterDeployment) {
+		clusterDeployment.Status.InstallRestarts = restarts
+	}
+}
+
 func WithClusterVersion(version string) Option {
 	return Generic(generic.WithLabel(constants.VersionMajorMinorPatchLabel, version))
 }
@@ -145,3 +159,24 @@ func WithHibernateAfter(dur time.Duration) Option {
 		clusterDeployment.Spec.HibernateAfter = &metav1.Duration{Duration: dur}
 	}
 }
+
+// WithAWSPlatform sets the specified aws platform on the supplied object.
+func WithAWSPlatform(platform *hivev1aws.Platform) Option {
+	return func(clusterDeployment *hivev1.ClusterDeployment) {
+		clusterDeployment.Spec.Platform.AWS = platform
+	}
+}
+
+// WithGCPPlatform sets the specified gcp platform on the supplied object.
+func WithGCPPlatform(platform *hivev1gcp.Platform) Option {
+	return func(clusterDeployment *hivev1.ClusterDeployment) {
+		clusterDeployment.Spec.Platform.GCP = platform
+	}
+}
+
+// WithAzurePlatform sets the specified azure platform on the supplied object.
+func WithAzurePlatform(platform *hivev1azure.Platform) Option {
+	return func(clusterDeployment *hivev1.ClusterDeployment) {
+		clusterDeployment.Spec.Platform.Azure = platform
+	}
+}
